/*
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (c) Copyright IBM Corp. 2008  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code 
 * created by IBM Corporation.  This sample code is part of the 
 * WebSphere Premises Server and is warranted to perform its 
 * intended function only if used un-modified.  If you modify 
 * this code then it is considered provided "AS IS", without 
 * warranty of any kind.  Notwithstanding the foregoing, IBM 
 * shall not be liable for any damages arising out of your use 
 * of the sample code, even if they have been advised of the 
 * possibility of such damages.
 */
package com.ibm.trackntrace.barcode.parser;



public abstract class AbstractBarCodeParser {

	protected String packagingTypeDigit = null; // also known as extension digit

	protected String serialNumber = null;

	protected String companyPrefix = null;

	protected int checkDigit = -1;

	protected String NDC = null;

	protected String EPC = null;

	protected String barcode = null;

	public enum encodingTypes {
		SSCC, SGTIN, UNKNOWN_ENCODING
	}

	public enum packagingTypes {
		UNIT, CASE, PALLET, SLEEVE, UNKNOWN_PACKAGE
	}

	public String getNDC() {
		return NDC;
	}

	public void setNDC(String ndc) {
		NDC = ndc;
	}

	abstract protected void parseBarCode() throws BarCodeParserException;

	public int getCheckDigit() {
		return checkDigit;
	}

	protected void setCheckDigit(int checkDigit) {
		this.checkDigit = checkDigit;
	}

	public String getCompanyPrefix() {
		return companyPrefix;
	}

	protected void setCompanyPrefix(String companyPrfix) {
		this.companyPrefix = companyPrfix;
	}

	public String getPackagingTypeDigit() {
		return packagingTypeDigit;
	}

	protected void setPackagingType(String packagingType) {
		this.packagingTypeDigit = packagingType;
	}

	public String getSerialNumber() {
		return serialNumber;
	}

	protected void setSerialNumber(String serialNumber) {
		this.serialNumber = serialNumber;
	}

	abstract protected void verifyCheckDigit(int inCheckDigit)
			throws BarCodeParserException;

	public String getEPC() {
		return EPC;
	}

	protected void setEPC(String epc) {
		EPC = epc;
	}

	public String getBarcode() {
		return barcode;
	}

	public void setBarcode(String barcode) {
		this.barcode = barcode;
	}
	
	public abstract packagingTypes getPackagingType();

	public static encodingTypes getEncodingType(String barcode) {

		if (barcode != null && barcode.length() >= 2) {
			String appID = barcode.substring(Constants.APP_ID_INDEX_START,
					Constants.APP_ID_PREFIX_INDEX_END);

			if (appID.equals(Constants.SSCC18Constants.AI_SSCC)) {
				return encodingTypes.SSCC;
			} else if (appID.equals(Constants.PharmaSGTIN14Constants.AI_SGTIN)) {
				return encodingTypes.SGTIN;
			} else
				return encodingTypes.UNKNOWN_ENCODING;
		} else
			return encodingTypes.UNKNOWN_ENCODING;

	}

}
