package com.ibm.annapolis.ndc_mapper;

import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.InitialContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.ibm.annapolis.ndc_mapper.utils.Constants;
import com.ibm.websphere.cache.DistributedMap;

/*
 * 
 * @author nick
 *
 * Document out:
 * - how XML file generated
 * - how to add entries to xml file -> part of GUI? (will need to cross-check for collisions)
 *  
 * TODO design load/reload scheme.
 * Goal is to use in MDB.  So how to re-load from XML file?
 * 1) Restart MDB - no
 * 2) Once a day - no timer avail, so 1st lookup of day will be longer.
 * 3) processing xml file takes ? time
 * 4) Rely on container MDB to call reload method.  Push trigger off to MDB
 * 5) What about mult instances?  Need to sychronize the reload mechanism.
 * 6) Remove static methods and have MDB create single instance of class.
 * 7) How handle errors in xml file - how report/alarm?
 */

public class NdcToIdentityUri {

	/**
	 * @param validNDCsFilePath
	 */
	public NdcToIdentityUri(String validNDCsFilePath) {
		this.xmlFileName = validNDCsFilePath;

	}

	/**
	 * constructor
	 */
	public NdcToIdentityUri() {

	}

	/**
	 * Logger for this class. Log level can be adjusted from the WAS admin
	 * console
	 */
	private static Logger theLogger = Logger.getLogger(NdcToIdentityUri.class
			.getName());

	/**
	 * variable that hold the class name of the object
	 */
	private String clsName = NdcToIdentityUri.class.getName();

	private DistributedMap ndcmap = null;

	// Where to load the xmlified ndc data from.
	// Hmmm, actual implementation would use getter()/setter() or include as
	// part on object creation.
	// Defering decision until actual implementaiton
	// "sample_data/All_Elements.xml";
	// "sample_data/10_Elements.xml";
	private String xmlFileName = "All_Elements.xml";

	private void helperPrintIfFound(String ndc, String serial, String packType) {
		String sRet = ndcToIdentityUri(ndc, serial, packType);
		if (sRet == null)
			theLogger.severe("ndc " + ndc + " was NOT found");
		else
			theLogger.info("ndc " + ndc + " was found (" + sRet + ")");
	}

	public static void main(String[] args) {

		String[] testndcs = { "", "351672527648", };

		NdcToIdentityUri foo = new NdcToIdentityUri();

		for (int i = 0; i < testndcs.length; i++)
			foo.helperPrintIfFound(testndcs[i], "junk", "junk");
	}

	private boolean loadMaps() {

		theLogger.entering(clsName, "loadMaps");
		boolean result = false;

		// lookup dynacache
		if (!lookupDynaCache()) {
				// lookup failed
			return result;

		}

		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document dom = db.parse(xmlFileName);
			Element root = dom.getDocumentElement();

			processNdcListElement(root);
			result = true;
			
			if( theLogger.getLevel() == Level.FINEST){
				dumpMap();
			}

		}
		// indicates a well-formedness error
		catch (Exception e) {
			e.printStackTrace();
			// TODO: complain to whom/where?
			theLogger
					.severe(xmlFileName
							+ " is not well-formed. error message is:"
							+ e.getMessage());
			return result;
		} finally {
			theLogger.exiting(clsName, "loadMaps");
		}

		return result;
	}

	/**
	 * Construct an IdentityUri from its components. The standard prefix is
	 * prepended.
	 * 
	 * @param labelCode -
	 *            the labeler code
	 * @param productCode -
	 *            the product code
	 * @param serial -
	 *            the serial number
	 * @return A string with in an Identity format
	 */
	private String mergeStrings(String labelCode, String productCode,
			String tradePackageSizeCode, String packType, String serial) {
		theLogger.entering(clsName, "mergeStrings", new Object[] { labelCode,
				productCode, tradePackageSizeCode, serial });

		String companyPrefix = Constants.NDC_INDICATOR + labelCode;
		String itemRef = packType + productCode + tradePackageSizeCode;

		theLogger.fine("CompanyPrefix=" + companyPrefix);
		theLogger.fine("ItemRef=" + itemRef);

		String epcUrn = Constants.IDENTITY_URI_PREFIX + companyPrefix + "."
				+ itemRef + "." + serial;
		theLogger.exiting(clsName, "mergeStrings", epcUrn);
		return epcUrn;
	}

	/**
	 * @param ndc
	 * @param serialnumber
	 * @param packType
	 * @return
	 */
	public String ndcToIdentityUri(String ndc, String serialnumber,
			String packType) {

		theLogger.entering(clsName, "ndcToIdentityUri", new Object[] { ndc,
				serialnumber });
		String sRet;

		// ndc and serial number must be non-null and non-empty strings
		if ((ndc == null) || (ndc.length() < 1) || (serialnumber == null)
				|| (serialnumber.length() < 1)) {
			// TODO: complain where/how?
			theLogger.severe("Either the NDC =" + ndc + " or SerialNumber="
					+ serialnumber + " passed in are invalid!");
			return null;
		}

		// Look in dynacache to see if there are entries in the map, if not load
		// the map.
		// Load the local map if not already loaded:
		if (lookupDynaCache()) {
			// lookup success
			// is there anything in the map?
			if (ndcmap.size() > 0) {
				if (theLogger.getLevel() == Level.FINEST) {
					theLogger.finest("Contents of Dynacache "
							+ Constants.NDC_DYNA_CACHE);
					dumpMap();
				}
			}
			else{
				//nothing in map
				theLogger.warning("No entries in Dynacache " + Constants.NDC_DYNA_CACHE);
				if (!loadMaps()) {
					// TODO: Complain to whom at where?
					return null;
				}
				
			}
		}
		
		// Variables to make it easier to perform the diff types of searches
		String lblCode = null; // labelCode
		String prodCode = null; // prodcutCode
		String searchtoken = null; // the token we search for. See class
									// comments at top of file.
		String tradePackageSizeCode = null; // the third segment of the NDC
											// number

		if (ndc.length() < 10) {
			theLogger
					.warning("The passed in NDC is invalid!. The length is less than 10. NDC="
							+ ndc);
			return null;
		}
		// -------------------------------------
		// Check for match using 5-4-1 pattern
		// See notes above about indexes - exported ndc database prepends
		// leading zero char
		lblCode = ndc.substring(0, 5);
		prodCode = ndc.substring(5, 9);
		tradePackageSizeCode = ndc.substring(9, 10);
		theLogger.finest("5-4-1: lblcode=" + lblCode + " prodCode=" + prodCode
				+ " packSize=" + tradePackageSizeCode);

		// ndc listing prepends leading 0 char
		// x is our internal seperator char
		searchtoken = "0" + lblCode + "x" + prodCode;
		theLogger.fine("Searching for \"" + searchtoken + "\", on " + lblCode
				+ "." + prodCode);
		sRet = (String) ndcmap.get(searchtoken);
		if (sRet != null) {
			sRet = mergeStrings(lblCode, prodCode, tradePackageSizeCode,
					packType, serialnumber);
			return sRet;
		}

		// -------------------------------------
		// Check for match using 5-3-2 pattern
		lblCode = ndc.substring(0, 5);
		prodCode = ndc.substring(5, 8);
		tradePackageSizeCode = ndc.substring(8, 10);

		theLogger.finest("5-4-2: lblcode=" + lblCode + " prodCode=" + prodCode
				+ " packSize=" + tradePackageSizeCode);

		// ndc listing prepends leading 0 char
		// x is our internal seperator char
		searchtoken = "0" + lblCode + "x" + prodCode;
		theLogger.finest("Searching for \"" + searchtoken + "\", on " + lblCode
				+ "." + prodCode);
		sRet = (String) ndcmap.get(searchtoken);
		if (sRet != null) {
			sRet = mergeStrings(lblCode, prodCode, tradePackageSizeCode,
					packType, serialnumber);
			return sRet;
		}
		// === Check 4-4-2 format
		// -------------------------------------
		// Check for match using 4-4-2 pattern
		lblCode = ndc.substring(0, 4);
		prodCode = ndc.substring(4, 8);
		tradePackageSizeCode = ndc.substring(8, 10);

		theLogger.finest("5-4-1: lblcode=" + lblCode + " prodCode=" + prodCode
				+ " packSize=" + tradePackageSizeCode);

		// ndc listing prepends leading 0 char
		// x is our internal seperator char
		searchtoken = "0" + lblCode + "x" + prodCode;
		theLogger.finest("Searching for \"" + searchtoken + "\", on " + lblCode
				+ "." + prodCode);
		sRet = (String) ndcmap.get(searchtoken);
		if (sRet != null) {
			sRet = mergeStrings(lblCode, prodCode, tradePackageSizeCode,
					packType, serialnumber);
			return sRet;
		}

		theLogger.exiting(clsName, "ndcToIdentityUri", sRet);
		return sRet;
	}

	// ---------------------------------------------------------------
	private void processNdcListElement(Element root) {
		theLogger.entering(clsName, "processNdcListElement");

		// Get list of <ndc> child elements:
		NodeList kids = root.getChildNodes();

		// iterate over the list, inserting into a map
		// Iterator iterator = kids.iterator();
		int count = 1;
	
		for (int i = 0; i < kids.getLength(); ++i) {
			Node kid = kids.item(i);
			if (kid != null && Constants.ELEMENT_NDC.equals(kid.getNodeName())) {
				theLogger.finest("element name=" + kid.getNodeName());

				// get attributes from element, primitive sanity check, insert
				// into map
				NamedNodeMap attribs = kid.getAttributes();

				for (int j = 0; j < attribs.getLength(); ++j) {
					theLogger.finest("Attrib Name= "
							+ attribs.item(j).getNodeName() + " Attrib Value= "
							+ attribs.item(j).getNodeValue());
				}
				String lblCode = null;
				String prodCode = null;

				Node lblNode = attribs.getNamedItem(Constants.ATTR_LABELER);
				if (lblNode != null) {
					lblCode = lblNode.getNodeValue();
					theLogger.finest("lblCode =" + lblCode);
				}

				Node prodNode = attribs.getNamedItem(Constants.ATTR_PRODUCT);

				if (prodNode != null) {
					prodCode = prodNode.getNodeValue();
					theLogger.finest("prodCode =" + prodCode);
				}

				// TODO: should check string lengths and character content
				// (numerics only)
				if ((lblCode != null) && (prodCode != null)) {
					String inserttoken = lblCode + "x" + prodCode;
					theLogger.finest("Inserting \"" + inserttoken + "\"");
					this.ndcmap.put(inserttoken, "");
				} else {
					// TODO: Complain to whom/where?
					theLogger.warning("Element error near line " + count
							+ ": Missing attributes?");
				}
				count++; // make a nicer error message in 60K lines, eh?
			}
		}
		theLogger.exiting(clsName, "processNdcListElement");
		return;
	}

	private boolean lookupDynaCache() {

		theLogger.entering(clsName, "lookupDynaCache");
		boolean result = false;
		// lookup dynacache
		try {
			InitialContext ic = new InitialContext();
			ndcmap = (DistributedMap) ic.lookup(Constants.NDC_DYNA_CACHE);
			if (ndcmap != null)
				result = true;
			else {
				theLogger.severe("Unable to access dynacache "
						+ Constants.NDC_DYNA_CACHE);
			}

		} catch (Exception e) {
			e.printStackTrace();
			theLogger.severe("Unable to create DistributedMap:"
					+ e.getMessage());
		}
		theLogger.exiting(clsName, "lookupDynaCache", result);
		return result;
	}

	private void dumpMap() {
		
		theLogger.entering(clsName, "dumpMap");
		Iterator it = ndcmap.entrySet().iterator();

		while (it.hasNext()) {
			Map.Entry pairs = (Map.Entry) it.next();
			theLogger.finest("key=" + pairs.getKey());
		}
		theLogger.exiting(clsName, "dumpMap");
	}
}