package com.ibm.trackntrace.actions;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.util.MessageResources;

import com.ibm.trackntrace.forms.FormData;
import com.ibm.trackntrace.usecases.AbstractUseCase;
import com.ibm.trackntrace.usecases.UseCaseException;
import com.ibm.trackntrace.usecases.data.CaseInfo;
import com.ibm.trackntrace.usecases.data.PalletInfo;
import com.ibm.trackntrace.usecases.data.PharmaUnitInfo;
import com.ibm.trackntrace.utils.Constants;


/**
 * @version 1.0
 * @author
 */
public abstract class BXActionBase extends Action {
	protected static Logger log = null;

	protected MessageResources msgs = null;

	protected static final DateFormat dateFormat = DateFormat
			.getDateInstance(DateFormat.SHORT);

	protected static final String REDISPLAY = "Redisplay"; //$NON-NLS-1$

	protected static final String ERROR_PAGE = "Error"; //$NON-NLS-1$

	protected static final String GLOBAL_ERROR_PAGE = "GlobalError"; //$NON-NLS-1$

	protected String getMessage(String key) {
		log.entering(getClass().getName(), "getMessage(key=" + key + ")"); //$NON-NLS-1$ //$NON-NLS-2$
		String text = null;
		if (msgs != null) {
			text = msgs.getMessage(key);
			if (text == null) {
				text = "Failed to find " + key + " key."; //$NON-NLS-1$ //$NON-NLS-2$
			}
		} else {
			text = "failed to find ApplicationResources bundle for messages."; //$NON-NLS-1$
		}
		log.exiting(getClass().getName(), "getMessage(key=" + key + ")", text); //$NON-NLS-1$ //$NON-NLS-2$
		return text;
	}

	protected boolean arrayDoesNotContain(String[] array, String target) {
		boolean result = true;

		if (array != null) {
			for (int i = 0; (i < array.length) && result; ++i) {
				if (target.equals(array[i])) {
					result = false;
				}
			}
		}

		return result;
	}

	protected String buildDisplayData(FormData formData, ActionErrors errors) {
		String target = null;

		String[] ndc = { "00000-000-00", "11111-111-11", "22222-222-22",
				"33333-333-33" };
		String[] cas = { "1", "3", "9", "12" };
		String[] unt = { "48", "150", "9", "144" };
		String[] des = { "Drug A 300mg 10 pills", "Drug B 300mg 20 pills",
				"Drug C 300mg 30 pills", "Drug E 300mg 40 pills" };

		String display = "<p><table class=\"scanData\">"
				+ "<tr><th width=70%>NDC</th>" + "<th width=15%>Cases</th>"
				+ "<th width=15%>Units </th></tr>"
				+ "<tr><th colspan=3>Item Description</th></tr>";
		final int j = 0x0001;
		for (int i = 0; i < ndc.length; i++) {
			int k = i & j;
			if (k == 0) {
				display = display + "<tr class=\"even\"><td>";
			} else {
				display = display + "<tr><td>";
			}
			display = display + ndc[i] + "</td><td>" + cas[i] + "</td><td>"
					+ unt[i] + "</td></tr>";
			if (k == 0) {
				display = display + "<tr class=\"even\">";
			} else {
				display = display + "<tr>";
			}
			display = display + "<td colspan=3>" + des[i] + "</td></tr>";
		}
		display = display + "</td></tr></table>";
		formData.setDisplayData(display);

		return target;
	}

	protected abstract ActionForward execute(ActionMapping mapping,
			FormData formData, HttpServletRequest request,
			HttpServletResponse response, ActionErrors errors) throws Exception;

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		ActionErrors errors = new ActionErrors();
		ActionForward forward = new ActionForward(); // return value

		msgs = getResources(request);
		try {
			if (form instanceof FormData) {
				FormData formData = (FormData) form;

				forward = execute(mapping, formData, request, response, errors);
			} else {
				errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
						Constants.ACTION_WRONG_FORM_3, request.getRequestURI(),
						FormData.class.getName(), form.getClass().getName()));
			}
		} catch (Throwable e) {
			log.log(Level.SEVERE,
					"Exception caught while processing request", e); //$NON-NLS-1$
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.ACTION_EXCEPTION_1, e.toString()));
		}

		if (forward == null) {
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.ACTION_NO_FORWARD_3, request.getRequestURI(),
					this.getClass().getName(), form.getClass().getName()));
			forward = mapping.findForward(ERROR_PAGE);
		}

		if (forward == null) {
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.ACTION_NO_ERROR_FORWARD_3, request
							.getRequestURI(), this.getClass().getName(), form
							.getClass().getName()));
			forward = mapping.findForward(GLOBAL_ERROR_PAGE);
		}

		if (!errors.isEmpty()) {
			Iterator msg_iter = errors.get();
			while (msg_iter.hasNext()) {
				ActionError msg = (ActionError) msg_iter.next();
				log
						.warning("Error recorded:  " + msg.getKey() + "; " + arrayToString(msg.getValues(), ", ")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
			saveErrors(request, errors);
		}

		log.exiting(getClass().getName(),
				"execute", "forward=" + forward.getName()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		return (forward);
	}

	private String arrayToString(Object[] values, String separator) {
		StringBuffer result = new StringBuffer();
		if ((values != null) && (values.length > 0)) {
			Object value = values[0];
			if (value != null) {
				result.append(value);
			} else {
				result.append("null"); //$NON-NLS-1$
			}
			for (int i = 1; i < values.length; ++i) {
				result.append(separator);
				value = values[i];
				if (value != null) {
					result.append(value);
				} else {
					result.append("null"); //$NON-NLS-1$
				}
			}
		} else {
			result.append("null"); //$NON-NLS-1$
		}
		return result.toString();
	}

	protected boolean removeBarCodeDetailsFromSession(
			Map<String, PharmaUnitInfo> tagDetailsMap, FormData formData,
			String barcode, String mapName) {

		log.entering(getClass().getName(), "removeFromTagDetails",
				new Object[] { tagDetailsMap, formData });

		boolean result = false;

		if (barcode != null) {
			if (tagDetailsMap.containsKey(barcode)) {
				log.finest("Removing barcode " + barcode + " from " + mapName);
				tagDetailsMap.remove(barcode);

				result = true;
			} else {
				log.severe("The " + mapName + " doesn't contain the key "
						+ barcode + " Existing entries "
						+ tagDetailsMap.toString());
				log
						.severe("Its possible the session might have expired since the "
								+ mapName
								+ " was created. The size of the map is"
								+ tagDetailsMap.size());
				formData
						.setLastError("Session might have expired. Please contact an administrator");

			}

		}
		log.exiting(getClass().getName(), "removeFromTagDetails", result);
		return result;

	}

	protected boolean removePalletBarCodeDetailsFromSession(
			Map<String, PalletInfo> tagDetailsMap, FormData formData,
			String barcode, String mapName) {

		log.entering(getClass().getName(),
				"removePalletBarCodeDetailsFromSession", new Object[] {
						tagDetailsMap, formData });

		boolean result = false;

		if (barcode != null) {
			if (tagDetailsMap.containsKey(barcode)) {
				log.finest("Removing barcode " + barcode + " from " + mapName);
				tagDetailsMap.remove(barcode);

				result = true;
			} else {
				log.severe("The " + mapName + " doesn't contain the key "
						+ barcode + " Existing entries "
						+ tagDetailsMap.toString());
				log
						.severe("Its possible the session might have expired since the "
								+ mapName
								+ " was created. The size of the map is"
								+ tagDetailsMap.size());
				formData
						.setLastError("Session might have expired. Please contact an administrator");

			}

		}
		log.exiting(getClass().getName(),
				"removePalletBarCodeDetailsFromSession", result);
		return result;

	}

	/**
	 * @param tagDetailsMap - session map to hold the data between requests
	 * @param barcode - scanned barcode
	 * @param urn - the scanned tag
	 * @param mapName - name of the session map that holds the barcode details
	 * @param formData - the formdata from the UI
	 * @param useCase - backend usecase object
	 * @param needParentData - is the parent tag information required on top of item info
	 * @return
	 */
	protected boolean addBarCodeDetailsToSession(
			Map<String, PharmaUnitInfo> tagDetailsMap, String barcode,
			String urn, String mapName, FormData formData,
			AbstractUseCase useCase, boolean needParentData) {

		log.entering(getClass().getName(), "addTagToSession", new Object[] {
				tagDetailsMap, barcode, urn, mapName });

		boolean result = true;
		try {
			if (barcode != null && urn != null && useCase != null) {

				if (!tagDetailsMap.containsKey(barcode)) {

					PharmaUnitInfo unitInfo = null;
					if( needParentData){
						unitInfo = useCase.getItemAndParentInfo(urn);
					}
					else{	
						unitInfo = useCase.getItemInfo(urn);
					}

					if (unitInfo == null) {
						result = false;
						log.severe("Received a NULL PharmaUnitInfo Object");
						formData
								.setLastError("Unable to retrieve barcode information. Backend not responsive.");
					} else {
						
						log.fine("Adding barcode= " + barcode
								+ " and PharmaUnitInfo =" + unitInfo.getNDC()
								+ " to the tagDetailsMap");
						tagDetailsMap.put(barcode, unitInfo);
					}
				} else {
					log.finest("The barcode " + barcode
							+ " already exists in the " + mapName);
				}
			}
			else{
				result = false;
			}
		} catch (UseCaseException e) {
			log.severe(e.getMessage());
			e.printStackTrace();
			result = false;
			formData
					.setLastError("Unable to retrieve barcode information. Backend not responsive.");
		}
		log.exiting(getClass().getName(), "addTagToSession", result);
		return result;
	}

	protected boolean addPalletBarCodeDetailsToSession(
			Map<String, PalletInfo> tagDetailsMap, String barcode, String urn,
			String mapName, FormData formData, AbstractUseCase useCase) {

		log.entering(getClass().getName(), "addPalletBarCodeDetailsToSession",
				new Object[] { tagDetailsMap, barcode, urn, mapName });

		boolean result = true;
		try {
			if (barcode != null && urn != null) {

				if (!tagDetailsMap.containsKey(barcode)) {

					PalletInfo palletInfo = useCase.getPalletDetails(urn);

					if (log.getLevel() == Level.FINEST) {
						ArrayList<CaseInfo> cases = palletInfo.getCases();
						if (cases != null && cases.size() > 1) {

							ArrayList<PharmaUnitInfo> units = cases.get(0)
									.getUnits();
							if (units != null && units.size() > 0) {
								log.fine("Adding barcode= " + barcode
										+ " and PalletInfo: NDC ="
										+ units.get(0).getNDC()
										+ " to the tagDetailsMap");
							}
						}
					}

					tagDetailsMap.put(barcode, palletInfo);
				} else {
					log.finest("The barcode " + barcode
							+ " already exists in the " + mapName);
				}
			}
		} catch (UseCaseException e) {
			log.severe(e.getMessage());
			e.printStackTrace();
			result = false;
			formData
					.setLastError("Please Contact administrator. Backend not responsive.");
		}
		log.exiting(getClass().getName(), "addPalletBarCodeDetailsToSession",
				result);
		return result;
	}

	@SuppressWarnings("unchecked")
	Map<String, PharmaUnitInfo> createTagDetailsMapInSession(
			HttpServletRequest request, String mapName) {

		log.entering(getClass().getName(), "createTagDetailsMapInSession",
				new Object[] { request, mapName });
		/*
		 * Create the TagDetailMap attribute in the session, if doesn't already
		 * exist. This attribute contains a map of scanned tags and associated
		 * product information.
		 */
		Map<String, PharmaUnitInfo> tagDetailsMap = null;
		tagDetailsMap = (Map<String, PharmaUnitInfo>) request.getSession()
				.getAttribute(mapName);
		if (tagDetailsMap == null) {
			log.fine(mapName + " doesn't exist in the session. sessionID="
					+ request.getSession().getId());

			tagDetailsMap = new LinkedHashMap<String, PharmaUnitInfo>();
			request.getSession().setAttribute(mapName, tagDetailsMap);
		} else {
			log
					.fine(mapName + " exists in the session "
							+ request.getSession().getId()
							+ " The current size of the map is "
							+ tagDetailsMap.size());
		}
		log.exiting(getClass().getName(), "createTagDetailsMapInSession",
				tagDetailsMap);

		return tagDetailsMap;
	}

	@SuppressWarnings("unchecked")
	Map<String, PalletInfo> createPalletTagDetailsMapInSession(
			HttpServletRequest request, String mapName) {

		log.entering(getClass().getName(),
				"createPalletTagDetailsMapInSession", new Object[] { request,
						mapName });
		/*
		 * Create the TagDetailMap attribute in the session, if doesn't already
		 * exist. This attribute contains a map of scanned tags and associated
		 * product information.
		 */
		Map<String, PalletInfo> tagDetailsMap = null;
		tagDetailsMap = (Map<String, PalletInfo>) request.getSession()
				.getAttribute(mapName);
		if (tagDetailsMap == null) {
			log.fine(mapName + " doesn't exist in the session. sessionID="
					+ request.getSession().getId());

			tagDetailsMap = new LinkedHashMap<String, PalletInfo>();
			request.getSession().setAttribute(mapName, tagDetailsMap);
		} else {
			log
					.fine(mapName + " exists in the session "
							+ request.getSession().getId()
							+ " The current size of the map is "
							+ tagDetailsMap.size());
		}
		log.exiting(getClass().getName(), "createPalletTagDetailsMapInSession",
				tagDetailsMap);

		return tagDetailsMap;
	}

	/**
	 * Is the scanned barcode a pallet barcode. Pallet barcodes are SSCC type.
	 * @param barcode
	 * @return
	 */
	protected boolean isPalletBarcode(String barcode){
		
		log.entering(getClass().getName(), "isPalletBarcode", barcode);
		
		boolean result = false;
		
		if( barcode.startsWith(com.ibm.annap.barcode.parser.Constants.SSCC18Constants.AI_SSCC)){
			result = true;
		}
		log.exiting(getClass().getName(), "isPalletBarcode", result);
		return result;
		
	}
}
