package com.ibm.trackntrace.actions;
import java.util.ArrayList;
import java.util.Map;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.ibm.trackntrace.forms.FormData;
import com.ibm.trackntrace.usecases.Damaged;
import com.ibm.trackntrace.usecases.UseCaseException;
import com.ibm.trackntrace.usecases.data.PharmaUnitInfo;
import com.ibm.trackntrace.utils.BarcodeUtils;
import com.ibm.trackntrace.utils.Constants;
import com.ibm.trackntrace.utils.RoleDefs;
import com.ibm.trackntrace.utils.RoleValidation;


/**
 * This class implments the action handlers for the Damaged use case.
 * The damaged use case has 2 scenrios: Damaged Common Flow and Damaged Rebuild Flow
 * In the Damaged common, flow, units/cases are scanned that the user
 * wants to mark as damaged, this class work with the backend to mark those cases as damaged.
 * 
 * In the Rebuild scenario, the user scans the pallet tag first, and then the cases
 * that they would like to keep, this class, fecthes the list of aggreagted cases to the
 * pallet in question from the backend, and computes the damaged cases as the difference
 * between the backend list and scanned cases. The computed damaged cases are presented
 * to the user for confirmation before being marked as damaged.
 *
 */
public class DamagedAction extends BXActionBase {

	private static final String MAIN_MENU = "GotoMainMenu";

	private static final String GET_ITEM_PAGE = "DamagedScanItem";

	private static final String GET_CONFIRM_PAGE = "DamagedConfirmCases";
	
	private static final String GET_REBUILD_CONFIRM_PAGE = "DamagedRebuildConfirmCases";

	private static final String GET_PALLET_PAGE = "DamagedRebuildScanPallet";

	private static final String GET_CASE_PAGE = "DamagedRebuildScanCase";

	private static final String TAG_DETAILS_MAP = "DamagedTagDetailsMap";

	private static final String CASE_LIST_FROM_BACKEND = "CaseListFromBackend";
	private static final String COMPUTED_DAMAGED_CASE_LIST = "ComputedDamagedCaseList";

	static {
		log = Logger.getLogger(Constants.LOG_DAMAGE);
	}

	/* This method is canned display data used for unit testing to bypass the backend.
	 * (non-Javadoc)
	 * @see bx.actions.BXActionBase#buildDisplayData(bx.forms.FormData, org.apache.struts.action.ActionErrors)
	 */
	protected String buildDisplayData(FormData formData, ActionErrors errors) {
		String target = null;

		String[] ndc = { "00000-000-00", "11111-111-11", "22222-222-22",
				"33333-333-33" };
		String[] cas = { "1", "3", "9", "12" };
		String[] unt = { "48", "150", "9", "144" };
		String[] des = { "Drug A 300mg 10 pills", "Drug B 300mg 20 pills",
				"Drug C 300mg 30 pills", "Drug E 300mg 40 pills" };

		String display = "<p><table class=\"scanData\">"
				+ "<tr><th width=70%>NDC</th>" + "<th width=15%>Cases</th>"
				+ "<th width=15%>Units </th></tr>"
				+ "<tr><th colspan=3>Item Description</th></tr>";
		final int j = 0x0001;
		for (int i = 0; i < ndc.length; i++) {
			int k = i & j;
			if (k == 0) {
				display = display + "<tr class=\"even\"><td>";
			} else {
				display = display + "<tr><td>";
			}
			display = display + ndc[i] + "</td><td>" + cas[i] + "</td><td>"
					+ unt[i] + "</td></tr>";
			if (k == 0) {
				display = display + "<tr class=\"even\">";
			} else {
				display = display + "<tr>";
			}
			display = display + "<td colspan=3>" + des[i] + "</td></tr>";
		}
		display = display + "</td></tr></table>";
		formData.setDisplayData(display);

		return target;
	}

	/**
	 * This method is invoked when a tag is scanned in the UI.
	 * This method converts the scanned barcode into a EPC URI
	 * and then fetches the details of the barcode from the backend
	 * before rendering it to the user.
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processTag(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {

		String target = null;
		String thisUrn = null;
		String thisTag = null;

		log.entering(getClass().getName(),
				"processTag", new Object[] { tagDetailsMap, formData, errors }); //$NON-NLS-1$

		// get current barcode scan
		thisTag = formData.getSerialNumber();

		if (thisTag != null) {
			// parse barcode to EPC URN
			thisUrn = BarcodeUtils.convertBarcodeToEPCpureID(thisTag, formData);
		} else {
			log.warning("Passed in barcode is null!");
			//formData.setLastError("Null Barcode scanned in");
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.NULL_BARCODE, this.getClass().getName()));
			target = REDISPLAY;
		}

		// save the stripped barcode as the case Tag.
		formData.setCaseTag(thisTag);

		// if barcode parse fails, return.
		if (thisUrn == null) {
			log
					.finest("convertBarcodeToEPCpureID returned an null urn for barcode="
							+ thisTag);
			target = REDISPLAY;
			return target;
		}

		// add the barcode to session data and get the barcode product details
		// from backend.
		Damaged damaged = new Damaged(formData.getStation(), formData.getBizLocation(), formData.getTimeZoneOffset());
		
		if (addBarCodeDetailsToSession(tagDetailsMap, thisTag, thisUrn,
				TAG_DETAILS_MAP, formData, damaged, true)) {
			// we don't know if this is a unit or a case, so set both to the
			// same value
			formData.setCaseCount(tagDetailsMap.size());
			// set the palletTag for the current tag in the formTag
			PharmaUnitInfo currentUnit = (PharmaUnitInfo) tagDetailsMap
					.get(thisTag);
			// For the damaged-common scenario, the pallet tag is retreived
			// from the backend, for the rebuild case, pallet tag is scanned in.
			if (currentUnit != null) {
				log.finest("Screen name = " + formData.getScreen());
				if (formData.getScreen().equals("DamagedScanItem")) {

					log.finest("setting pallet tag as "
							+ currentUnit.getRootParentURN()
							+ " for unit tag = " + currentUnit.getUnitURN()
							+ " barcode= " + thisTag);
					formData.setPalletTag(currentUnit.getRootParentURN());
				}

			} else {
				log.warning("Unable to set pallet tag, current "
						+ "unit is null in the tag details map for"
						+ "barcode = " + thisTag);
			}

			// render the product data for the barcode.
			buildDisplayData(tagDetailsMap, formData, errors);
		} else {
			log
					.fine("addBarCodeDetailsToSession failed! setting display to current screen");
			target = REDISPLAY;
		}
		log.exiting(getClass().getName(), "processTag", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	/**
	 * This method is invoked when a pallet tag is scanned by user in the rebuild scenario,
	 * this method fetches the aggregated case list for the scanned pallet from the backend.
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @param request
	 * @return
	 */
	protected String processPalletTag(
			Map<String, PharmaUnitInfo> tagDetailsMap, FormData formData,
			ActionErrors errors, HttpServletRequest request) {

		String target = null;
		String thisUrn = null;
		String thisTag = null;

		log.entering(getClass().getName(), "processPalletTag", new Object[] {
				tagDetailsMap, formData, errors, request }); //$NON-NLS-1$

		// get current barcode scan
		thisTag = formData.getSerialNumber();

		if (thisTag != null) {
			
			if( !isPalletBarcode(thisTag)){
				//formData.setLastError("The scanned tag is not a pallet barcode");
				errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.PALLET_EXPECTED, this.getClass().getName()));
				target = REDISPLAY;
				return target;
			}
			
			// parse barcode to EPC URN
			thisUrn = BarcodeUtils.convertBarcodeToEPCpureID(thisTag, formData);
		} else {
			log.warning("Passed in barcode is null!");
			//formData.setLastError("Null Barcode scanned in");
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.NULL_BARCODE, this.getClass().getName()));
			target = REDISPLAY;
		}

		// save the stripped barcode as the pallet Tag.
		formData.setSerialNumber(thisTag);
		formData.setPalletTag(thisTag);
		formData.setPalletUrn(thisUrn);

		// if barcode parse fails, return.
		if (thisUrn == null) {
			log
					.finest("convertBarcodeToEPCpureID returned an null urn for barcode="
							+ thisTag);
			target = REDISPLAY;
			return target;
		}
		// send the pallet barcode to the backend and fetched the corresponding
		// case list.
		ArrayList<String> caseUrnListForPallet = null;
		try {
			Damaged damaged = new Damaged(formData.getStation(), formData.getBizLocation(), formData.getTimeZoneOffset());
			caseUrnListForPallet = damaged.rebuildPallet_OnPalletTag(thisUrn);
		} catch (UseCaseException e) {
			log
					.severe("Caught exception trying to retrieve cases for pallet barcode "
							+ thisTag);
			e.printStackTrace();
			log.severe("Unable to retrieve case list from backend !"
					+ thisTag);
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.BACKEND_ERROR, this.getClass().getName()));
		}
		// save the case urn list in the session, we will use it later
		// to compare with the list of scanned tags by the user.
		if (caseUrnListForPallet != null) {
			if (log.isLoggable(java.util.logging.Level.FINE)) {
				for (int i = 0; i < caseUrnListForPallet.size(); ++i) {
					log.fine("Retreived case urn = "
							+ caseUrnListForPallet.get(i)
							+ " for pallet barcode " + thisTag + " pallet urn="
							+ thisUrn);
				}
			}
			if (caseUrnListForPallet.size() == 0) {
				log.warning("No cases in the pallet barcode " + thisTag);
			}

			request.getSession().setAttribute(CASE_LIST_FROM_BACKEND,
					caseUrnListForPallet);
		}

		log.exiting(getClass().getName(),
				"processPalletTag", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	/**
	 * This method deals with the 'confirm' button press
	 * for Damaged-common scenarios, the scanned cases are sent to 
	 * the backend to be marked as damaged.
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processConfirm(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {
		String target = null;
		log.entering(getClass().getName(), "processConfirm", formData); //$NON-NLS-1$

		// submit the tag list to the backend.
		try {
			Damaged damaged = new Damaged(formData.getStation(), formData.getBizLocation(), formData.getTimeZoneOffset());
			Object[] barcodes = tagDetailsMap.keySet().toArray();
			for (int i = 0; i < barcodes.length; ++i) {
				String barcode = (String) barcodes[i];
				PharmaUnitInfo unitInfo = tagDetailsMap.get(barcode);
				log.finest("Retrieved unitInfo=" + unitInfo + " for barcode "
						+ barcode);

				if (unitInfo != null
						&& damaged.confirmDamage(unitInfo.getParentURN())) {
					log
							.fine("Damage confirmed with backend for URN "
									+ barcode);
					// clear tag list in session.
					this.removeBarCodeDetailsFromSession(tagDetailsMap, formData, barcode, TAG_DETAILS_MAP);
				} else {
					log
							.severe("Unable to confirm damage with backend for barcode = "
									+ barcode);
					formData
							.setLastError("Unable to mark cases as damaged. Please report to administrator");
					target = REDISPLAY;
					break;
				}
			}
		} catch (UseCaseException e) {
			log.severe("Caught use case exception during Sampling. "
					+ e.getMessage());
			e.printStackTrace();
			target = REDISPLAY;
			formData
					.setLastError("Serious error occured. Please report to Administrator");

		}

		log.exiting(getClass().getName(), "processConfirm", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	/**
	 * This method implments the 'Undo' button functionality.
	 * The last scanned case is removed from the list of scanned cases.
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processUndo(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {
		String target = GET_ITEM_PAGE;
		String lastBarcode = null;
		log
				.entering(
						getClass().getName(),
						"processUndo", new Object[] { tagDetailsMap, formData, errors }); //$NON-NLS-1$

		if (tagDetailsMap == null || formData == null) {
			log
					.severe("Input arguments are not as expected. Null argument(s) passed in");
			return target;
		}
		// get the last barcode from the session map
		if (tagDetailsMap.size() != 0) {
			// get the last entry in the map, this is a LinkedHashMap, so the
			// keys are ordered by insertion
			Object[] barcodes = tagDetailsMap.keySet().toArray();
			if (barcodes.length > 0) {
				lastBarcode = (String) barcodes[barcodes.length - 1];
				if (lastBarcode != null) {
					// remove the details from the session
					if (!removeBarCodeDetailsFromSession(tagDetailsMap,
							formData, lastBarcode, TAG_DETAILS_MAP)) {
						target = REDISPLAY;
						return target;
					} else {
						// get new barcode list
						Object[] newBarcodeList = tagDetailsMap.keySet()
								.toArray();
						if (formData.getCaseCount() > 0) {
							formData.setCaseCount(tagDetailsMap.size());
							if (newBarcodeList.length > 0) {
								String caseTag = (String) newBarcodeList[newBarcodeList.length - 1];
								formData.setCaseTag(caseTag);
								PharmaUnitInfo caseTagDetails = (PharmaUnitInfo) tagDetailsMap
										.get(caseTag);

								// Damaged has 2 use cases, that share this
								// code.
								// For damaged common, there is no case list in
								// the session.
								// the pallet tag is computed URN.
								// For the rebuilt scenario, the pallet tag is
								// scanned in.t
								// by the screen name

								if (formData.getScreen().equalsIgnoreCase(
										"DamagedScanItem")) {
									formData.setPalletTag(caseTagDetails
											.getRootParentURN());
								}

								target = GET_CASE_PAGE;
							} else {
								// no more barcodes left in list.
								formData.setPalletTag("");
								formData.setCaseTag("");
							}

						} else {

							target = GET_ITEM_PAGE;
						}
						// change display to reflect the undo.
						buildDisplayData(tagDetailsMap, formData, errors);
					}
				}
			}
		} else {
			formData.setCaseTag("");
			formData.setPalletTag("");
			formData.setCaseCount(0);
			
			log.warning("Nothing to undo!");
			//formData.setLastError("Nothing to undo!");
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.NOTHING_TO_UNDO, this.getClass().getName()));
			
		}

		log.exiting(getClass().getName(), "processUndo", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	/**
	 * This method implements the 'confirm' button press for rebuild scenario
	 * the computed damaged case list is sent to the backend to be marked as damaged.
	 * @param damagedCaseListURN
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processConfirmRebuildCases(ArrayList<String> damagedCaseListURN, FormData formData,
			ActionErrors errors) {
		String target = null;
		log.entering(getClass().getName(),
				"processConfirmRebuildCases", formData);
		
		if( damagedCaseListURN != null ){
			
			
			try {
				Damaged damaged = new Damaged(formData.getStation(), formData.getBizLocation(), formData.getTimeZoneOffset());
				damaged.rebuildPallet_OnConfirm(formData.getPalletUrn(), damagedCaseListURN, null);
			}catch( UseCaseException e){
				e.printStackTrace();
				//formData.setLastError("Unable to confirm damage with backend!");
				errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.CONFIRM_FAILED, this.getClass().getName()));
				target = GET_ITEM_PAGE;
			}
		}else{
			//formData.setLastError("No Damaged Case List");
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.NOTHING_TO_CONFIRM, this.getClass().getName()));
			log.severe("Damaged case List is null!");
			target = GET_ITEM_PAGE;
		}

		log.exiting(getClass().getName(),
				"processConfirmRebuildCases", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	/**
	 * Implements the 'Done' button functionality.
	 * For the rebuild scenarios, the damaged cases list is computed here.
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @param request
	 * @return
	 */
	protected String processScanDone(Map<String, PharmaUnitInfo> tagDetailsMap,
			FormData formData, ActionErrors errors, HttpServletRequest request) {
		String target = null;
		log.entering(getClass().getName(), "processScanDone", formData); //$NON-NLS-1$
		
		if( tagDetailsMap != null && tagDetailsMap.size() == 0){
			// no tags have been scanned yet.
			log.warning("No tags in the " + TAG_DETAILS_MAP);
			//formData.setLastError("No tags have been scanned yet!");
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.NO_TAGS_SCANNED, this.getClass().getName()));
			target = GET_ITEM_PAGE;
			return target;
		}

		// The user preses the confirm button after done, to mark cases as
		// damaged.
		// the logic to mark cases as damaged is in the backend.

		// For the Rebuild scenario, compute the damaged cases list by comparing
		// the scanned list with the backend list saved in the session variable
		// CASE_LIST_FROM_BACKEND
		log.finest("Screen name=" + formData.getScreen());
		if (formData.getScreen().equals("DamagedRebuildScanCase")) {
			// get the case list saved in the session
			@SuppressWarnings("unchecked")
			ArrayList<String> backendList = (ArrayList<String>) request
					.getSession().getAttribute(CASE_LIST_FROM_BACKEND);

			// compute damaged cases, initialize with backend values
			ArrayList<String> computedDamagedCases = new ArrayList<String>(
					backendList);

			// now lets remove the items from the computed list that have been
			// scanned
			// by users as cases that they would like to keep, the remainder
			// will
			// make up the computed damaged list
			Object[] scannedCaseBarcodes = tagDetailsMap.keySet().toArray();

			for (int i = 0; i < scannedCaseBarcodes.length; ++i) {
				String barcode = (String) scannedCaseBarcodes[i];
				// convert the barcode to an URN
				String urn = null;
				urn = BarcodeUtils.convertBarcodeToEPCpureID(barcode, formData);
				if (urn != null) {

					if (!computedDamagedCases.remove(urn)) {
						// the urn is not the backend list
						log
								.severe("The scanned barcode "
										+ barcode
										+ " is not part of"
										+ " the backend list of aggregated cases for the pallet "
										+ formData.getPalletTag()
										+ " pallet URN = " + urn
										+ " Valid cases URNs are :"
										+ backendList.toString());
						formData
								.setLastError("The user may not add new cases during damage rebuild !");
						target = GET_CASE_PAGE;
						break;
					}
				}
			}
			if (computedDamagedCases.size() > 0) {
				// there are damaged cases, display them
				buildDisplayCaseList(computedDamagedCases, formData, errors);
				//save the computed damaged cases in the session.
				//upon confirmation from the user, send them to backend for disaggr
				request.getSession().setAttribute(COMPUTED_DAMAGED_CASE_LIST, computedDamagedCases);
				target = GET_REBUILD_CONFIRM_PAGE;
			} else {
				// no damaged cases, pallet fully rebuilt.
				//formData.setLastError("No Damaged Cases!");
				errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.NO_DAMAGED_CASES, this.getClass().getName()));
				target = GET_ITEM_PAGE;
			}
		}

		log
				.exiting(getClass().getName(),
						"processScanDone", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;

	}

	/**
	 * clears form data.
	 * @param formData
	 */
	protected void clearForm(FormData formData) {
		formData.setTagList(null);
		formData.setSerialNumber(null);
		formData.setDisplayData(null);
		formData.setEpcUrn(null);
		formData.setPalletTag(null);
		formData.setPalletUrn(null);
		formData.setCaseTag(null);
		formData.setCaseUrn(null);
		formData.setUnitTag(null);
		formData.setUnitUrn(null);
		formData.setOrderNumber(null);
		formData.setLastOrderNum(null);
		formData.setCaseCount(0);
		formData.setUnitCount(0);
		formData.setChildCount(0);
		formData.setPalletNum(0);
		formData.setPalletCnt(0);
		formData.setNumTagsScanned(0);
		formData.setNeedTagCounter(false);
	}

	/*This method invoke different process method based on command and screen name.
	 *  (non-Javadoc)
	 * @see bx.actions.BXActionBase#execute(org.apache.struts.action.ActionMapping, bx.forms.FormData, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, org.apache.struts.action.ActionErrors)
	 */
	@SuppressWarnings("unchecked")
	protected ActionForward execute(ActionMapping mapping, FormData formData,
			HttpServletRequest request, HttpServletResponse response,
			ActionErrors errors) throws Exception {
		ActionForward forward = new ActionForward(); // return value
		String target = null;
		String screen = formData.getScreen();
		String command = formData.getCommand();
	

		log.entering(getClass().getName(), "execute", formData); //$NON-NLS-1$

		// Strip barcode of prefix
		formData.setSerialNumber(BarcodeUtils.stripBarcodePrefix(formData
				.getSerialNumber()));

		// get the barcode details from the session
		Map<String, PharmaUnitInfo> tagDetailsMap = createTagDetailsMapInSession(
				request, TAG_DETAILS_MAP);

		log.finest("Command = " + command);

		if (RoleValidation.validate(request, RoleDefs.DAMAGED) == false) {
			target = "GotoMainMenu";

		} else if (command.equals("GotItem")) { //$NON-NLS-1$
			target = processTag(tagDetailsMap, formData, errors);
			if (target == null) {
				target = GET_ITEM_PAGE;
			} else if (target.equals(REDISPLAY)) {
				target = GET_ITEM_PAGE;
			}

		} else if (command.equals("GotPallet")) { //$NON-NLS-1$
			target = processPalletTag(tagDetailsMap, formData, errors, request);
			if (target == null) {
				target = GET_CASE_PAGE;
			} else if (target.equals(REDISPLAY)) {
				target = GET_PALLET_PAGE;
			}

		} else if (command.equals("GotCase")) { //$NON-NLS-1$
			target = processTag(tagDetailsMap, formData, errors);
			if (target == null) {
				target = GET_CASE_PAGE;
			} else if (target.equals(REDISPLAY)) {
				target = GET_CASE_PAGE;
			}

		} else if (command.equals("Done")) { //$NON-NLS-1$
			target = processScanDone(tagDetailsMap, formData, errors, request);
			if (target == null) {
				if(screen.equalsIgnoreCase("DamagedRebuildConfirm")){
					target = GET_REBUILD_CONFIRM_PAGE;
				}
				else
					target = GET_CONFIRM_PAGE;
			}

		} else if (command.equals("Undo")) { //$NON-NLS-1$
			target = processUndo(tagDetailsMap, formData, errors);
			if (target == null) {
				//formData.setLastError("processUndo failed.");
				errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.UNDO_FAILED, this.getClass().getName()));
				target = GET_ITEM_PAGE;
			}

		} else if (command.equals("Rebuild")) { //$NON-NLS-1$
			clearForm(formData);
			target = GET_PALLET_PAGE;

		} else if (command.equals("Confirm")) { //$NON-NLS-1$

			if(screen.equalsIgnoreCase("DamagedRebuildConfirm")){
				ArrayList<String> computedDamagedCasesList = 
					(ArrayList<String>) request.getSession().getAttribute(COMPUTED_DAMAGED_CASE_LIST);
				
				this.processConfirmRebuildCases(computedDamagedCasesList, formData, errors);
			}
			else {
				processConfirm(tagDetailsMap, formData, errors);
			}
			
			clearForm(formData);
			target = GET_ITEM_PAGE;

		} else if (command.equals("Cancel")) { //$NON-NLS-1$
			clearForm(formData);
			target = MAIN_MENU;

		} else if (command.equals("MainMenu")) { //$NON-NLS-1$
			target = MAIN_MENU;
		}

		if (target != null) {
			forward = mapping.findForward(target);
			request.setAttribute(Constants.FORM_DATA_ATTR, formData);
		} else {
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.UNKNOWN_COMMAND_4, request.getRequestURI(),
					screen, command, this.getClass().getName()));
		}

		log.exiting(getClass().getName(),
				"execute", "target=" + target + "; forward=" + forward); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

		// Finish with
		return (forward);
	}

	/**
	 * Build the display string to render the barcode product information.
	 * 
	 * @param map
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String buildDisplayData(Map<String, PharmaUnitInfo> map,
			FormData formData, ActionErrors errors) {

		log.entering(getClass().getName(), "buildDisplayData", new Object[] {
				map, formData, errors });

		String target = null;

		ArrayList<String> ndc = new ArrayList<String>();
		ArrayList<String> des = new ArrayList<String>();

		Object[] unitInfoArray = (Object[]) map.values().toArray();

		if (unitInfoArray.length != 0) {

			for (int i = 0; i < unitInfoArray.length; ++i) {
				PharmaUnitInfo unit = (PharmaUnitInfo) unitInfoArray[i];
				ndc.add(unit.getNDC());
				des.add(unit.getProductName());
			}

			String display = "<p><table class=\"scanData\">"
					+ "<tr><th width=70%>NDC</th>" + "</tr>"
					+ "<tr><th colspan=3>Item Description</th></tr>";
			final int j = 0x0001;

			for (int i = 0; i < unitInfoArray.length; i++) {
				int k = i & j;
				if (k == 0) {
					display = display + "<tr class=\"even\"><td>";
				} else {
					display = display + "<tr><td>";
				}
				display = display + ndc.get(i) + "</td></tr>";

				if (k == 0) {
					display = display + "<tr class=\"even\">";
				} else {
					display = display + "<tr>";
				}
				display = display + "<td colspan=3>" + des.get(i)
						+ "</td></tr>";
			}
			display = display + "</td></tr></table>";
			formData.setDisplayData(display);
		} else {
			// nothing to display
			log
					.warning("TagDetailsMap is empty! Setting form unit information display data to null");
			formData.setDisplayData("");
		}

		log.exiting(getClass().getName(), "buildDisplayData", target);
		return target;
	}

	/**
	 * Build the display string to render the barcode product information.
	 * 
	 * @param map
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String buildDisplayCaseList(ArrayList<String> caseURNs,
			FormData formData, ActionErrors errors) {

		log.entering(getClass().getName(), "buildDisplayCaseList", new Object[] {
				caseURNs, formData, errors });

		String target = null;
		Damaged damaged = null ; 
		try {
			damaged = new Damaged(formData.getStation(), formData.getBizLocation(), formData.getTimeZoneOffset());
			if (caseURNs != null && caseURNs.size() != 0) {

				String display = "<p><table class=\"scanData\">"
						+ "<tr><th width=70%>Damaged Case URN</th>"
						+ "<th width=70%>NDC</th>" + "</tr>";

				final int j = 0x0001;

				for (int i = 0; i < caseURNs.size(); i++) {
					int k = i & j;
					if (k == 0) {
						display = display + "<tr class=\"even\"><td>";
					} else {
						display = display + "<tr><td>";
					}
					display = display
							+ caseURNs.get(i)
							+ "</td></tr><tr><td>"
							+ damaged.getItemInfo(caseURNs.get(i))
									.getNDC() + "</td></tr>";

					if (k == 0) {
						display = display + "<tr class=\"even\">";
					} else {
						display = display + "<tr>";
					}
				}
				display = display + "</td></tr></table>";
				formData.setDisplayData(display);
			} else {
				// nothing to display
				log
						.warning("CaseBarcodesList is empty! Setting form unit information display data to null");
				formData.setDisplayData("");
			}

		} catch (UseCaseException e) {
			e.printStackTrace();
			//formData.setLastError("Unable to display damaged case list");
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(Constants.DISPLAY_ERROR, this.getClass().getName()));
			target=GET_ITEM_PAGE;
		}

		log.exiting(getClass().getName(), "buildDisplayCaseList", target);
		return target;
	}
}
