package com.ibm.trackntrace.actions;

import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.ibm.trackntrace.forms.FormData;
import com.ibm.trackntrace.usecases.Receiving;
import com.ibm.trackntrace.usecases.UseCaseException;
import com.ibm.trackntrace.usecases.data.OrderInfo;
import com.ibm.trackntrace.usecases.data.PalletInfo;
import com.ibm.trackntrace.usecases.data.PharmaUnitInfo;
import com.ibm.trackntrace.utils.BarcodeUtils;
import com.ibm.trackntrace.utils.Constants;
import com.ibm.trackntrace.utils.RoleDefs;
import com.ibm.trackntrace.utils.RoleValidation;


public class ReceivingAction extends BXActionBase {

	private static final String MAIN_MENU = "GotoMainMenu";

	private static final String GET_PALLET_PAGE = "ReceivingScanPallet";

	private static final String SHOW_PALLET_PAGE = "ReceivingShowPallet";

	private static final String ORDER_WARNING_PAGE = "ReceivingOrderWarning";

	private static final String PREVIOUS_ORDER = "PreviousOrder";

	static {
		log = Logger.getLogger(Constants.LOG_RECEIVING);
	}

	/**
	 * Processes a pallet tag being scanned from the UI. The Pallet tag is saved
	 * in form data and page is navigated to the get cases for pallet screen
	 * 
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processPalletTag(FormData formData, ActionErrors errors,
			HttpServletRequest request) {
		String target = null;
		log.entering(getClass().getName(), "processPalletTag", formData); //$NON-NLS-1$

		// get current barcode scan
		String thisTag = formData.getSerialNumber();
		String thisUrn = null;

		if (thisTag != null) {
			// parse barcode to EPC URN
			thisUrn = BarcodeUtils.convertBarcodeToEPCpureID(thisTag, formData);
		} else {
			log.warning("Passed in barcode is null!");
			formData.setLastError("Null Barcode scanned in");
			target = REDISPLAY;
		}

		// get the counts and transfer order number from the backend

		try {
			String userAgentID = formData.getStation();
			Receiving recv = null;
			PalletInfo palletInfo = null;

			if (userAgentID != null) {
				recv = new Receiving(userAgentID, formData.getBizLocation(), formData.getTimeZoneOffset());
			} else {
				log
						.severe("No station ID supplied. Using default value HandHeld1");
				recv = new Receiving("HandHeld1", formData.getBizLocation(), formData.getTimeZoneOffset());
			}

			if (thisUrn != null)
				palletInfo = recv.getPalletDetails(thisUrn);
			else {
				log.severe("URN is null for pallet barcode " + thisTag);
				formData.setLastError("Null URN for scanned barcode");
				target = REDISPLAY;
				return target;
			}

			if (palletInfo != null) {
				OrderInfo previousOrder = (OrderInfo) request.getSession()
						.getAttribute(PREVIOUS_ORDER);

				if (previousOrder != null) {

					if (!previousOrder.getOrderID().equalsIgnoreCase(
							palletInfo.getOrder().getOrderID())) {
						// scanned pallet has different order than previous
						// order
						log.fine("Previous Order #="
								+ previousOrder.getOrderID());
						log.fine("pallets received so far "
								+ previousOrder.getReceivedPalletEPCs().size());
						log.fine("expected pallets for previous order "
								+ previousOrder.getOrderID() + " is "
								+ previousOrder.getExpectedPalletEPCs().size());
						if (previousOrder.getReceivedPalletEPCs().size() < previousOrder
								.getExpectedPalletEPCs().size()) {
							// we are expecting more pallets for the previous
							// order
							// warn user..
							log
									.warning("Scanned pallet's order number "
											+ palletInfo.getOrder()
													.getOrderID()
											+ " is different from previous order number "
											+ formData.getOrderNumber()
											+ " More pallets expected in the previous order. Scanned so far = "
											+ previousOrder
													.getReceivedPalletEPCs()
													.size()
											+ " Expected pallets for Order "
											+ formData.getOrderNumber()
											+ " is "
											+ previousOrder
													.getExpectedPalletEPCs()
													.size());

							target = ORDER_WARNING_PAGE;
							return target;
						} else {
							log.info("Previous Order "
									+ previousOrder.getOrderID()
									+ " has been fully received!. "
									+ "Starting on new Order "
									+ palletInfo.getOrder().getOrderID());
							// user scanned a pallet with a different Order ID,
							// but its all good as the previous order was
							// completly received.
							// Save the new Order in the session.
							request.getSession().setAttribute(PREVIOUS_ORDER,
									palletInfo.getOrder());
							formData.setPalletNum(palletInfo.getOrder()
									.getReceivedPalletEPCs().size());
							formData.setPalletCnt(palletInfo.getOrder()
									.getExpectedPalletEPCs().size());
							formData.setOrderNumber(palletInfo.getOrder().getOrderID());

						}
					} else {
						// previous order is same as the scanned pallets order
						// we don't check to see if they are receving more
						// pallets than whats expected.
						// set form variable for display of recv/expected
						// pallets
						log.info("Previous order " + previousOrder.getOrderID()
								+ " is the same as the current "
								+ "scanned pallet's order ="
								+ palletInfo.getOrder().getOrderID());
						formData.setPalletNum(previousOrder
								.getReceivedPalletEPCs().size());
						formData.setPalletCnt(previousOrder
								.getExpectedPalletEPCs().size());
						formData.setOrderNumber(previousOrder.getOrderID());

					}
				} else {
					// previous order is null.
					// This must be the first pallet scanned in this session.
					log.info("Previous order is null. This order "
							+ palletInfo.getOrder().getOrderID()
							+ " must be the first one scanned this session");
					formData.setPalletNum(palletInfo.getOrder()
							.getReceivedPalletEPCs().size());
					formData.setPalletCnt(palletInfo.getOrder()
							.getExpectedPalletEPCs().size());
					formData.setOrderNumber(palletInfo.getOrder().getOrderID());
					request.getSession().setAttribute(PREVIOUS_ORDER,
							palletInfo.getOrder());
				}

				// set pallet and count information in the form for display
				formData.setPalletTag(thisTag);
				formData.setPalletUrn(thisUrn);
				formData.setCaseCount(palletInfo.getCasesPerPallet());
				formData.setUnitCount(palletInfo.getUnitsPerPallet());

				// if we have cases and units display their information on the
				// GUI.
				if (palletInfo.getCasesPerPallet() >= 1
						&& palletInfo.getUnitsPerPallet() >= 1) {
					PharmaUnitInfo unit = palletInfo.getCases().get(0)
							.getUnits().get(0);

					if (unit != null)
						buildDisplayData(palletInfo.getCases().get(0)
								.getUnits().get(0), formData, errors);
					else {
						formData
								.setLastError("Unable to display pallet information. Please report to administrator");
						log.severe("Unit information is null for pallet "
								+ formData.getPalletTag());
					}

				}

			}
		} catch (UseCaseException e) {
			e.printStackTrace();
			formData.setLastError("Backend error. Report to Administrator");
		}

		log
				.exiting(
						getClass().getName(),
						"processPalletTag", "target=" + target + " formData =" + formData.toString()); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	protected String processOrderWarningConfirm(FormData formData,
			ActionErrors errors, HttpServletRequest request) {
		String target = SHOW_PALLET_PAGE;
		log.entering(getClass().getName(), "processOrderWarningConfirm",
				formData);
		System.out.println(formData.toString());
		// get current barcode scan
		String thisTag = formData.getSerialNumber();
		String thisUrn = null;

		if (thisTag != null) {
			// parse barcode to EPC URN
			thisUrn = BarcodeUtils.convertBarcodeToEPCpureID(thisTag, formData);
		} else {
			log.warning("Passed in barcode is null!");
			formData.setLastError("Null Barcode scanned in");
			target = REDISPLAY;
		}

		// get the counts and transfer order number from the backend

		try {
			String userAgentID = formData.getStation();
			Receiving recv = null;
			PalletInfo palletInfo = null;

			if (userAgentID != null) {
				recv = new Receiving(userAgentID, formData.getBizLocation(), formData.getTimeZoneOffset());
			} else {
				log
						.severe("No station ID supplied. Using default value HandHeld1");
				recv = new Receiving("HandHeld1", formData.getBizLocation(), formData.getTimeZoneOffset());
			}

			if (thisUrn != null)
				palletInfo = recv.getPalletDetails(thisUrn);
			else {
				log.severe("URN is null for pallet barcode " + thisTag);
				formData.setLastError("Null URN for scanned barcode");
				target = REDISPLAY;
				return target;
			}

			formData.setPalletNum(0);
			formData.setPalletCnt(palletInfo.getOrder().getExpectedPalletEPCs()
					.size());
			request.getSession().setAttribute(PREVIOUS_ORDER,
					palletInfo.getOrder());

			// set pallet and count information in the form for display
			formData.setPalletTag(thisTag);
			formData.setPalletUrn(thisUrn);
			formData.setCaseCount(palletInfo.getCasesPerPallet());
			formData.setUnitCount(palletInfo.getUnitsPerPallet());

			// if we have cases and units display their information on the
			// GUI.
			if (palletInfo.getCasesPerPallet() >= 1
					&& palletInfo.getUnitsPerPallet() >= 1) {
				PharmaUnitInfo unit = palletInfo.getCases().get(0).getUnits()
						.get(0);

				if (unit != null)
					buildDisplayData(palletInfo.getCases().get(0).getUnits()
							.get(0), formData, errors);
				else {
					formData
							.setLastError("Unable to display pallet information. Please report to administrator");
					log.severe("Unit information is null for pallet "
							+ formData.getPalletTag());
				}

			}

		} catch (UseCaseException e) {
			e.printStackTrace();
		}

		log.exiting(getClass().getName(), "processOrderWarningConfirm",
				formData);
		return target;

	}

	protected String processConfirm(FormData formData, ActionErrors errors,
			HttpServletRequest request) {
		String target = null;
		log.entering(getClass().getName(), "processConfirm", formData);

		try {
			String userAgentID = formData.getStation();
			Receiving recv = null;

			if (userAgentID != null) {
				recv = new Receiving(userAgentID, formData.getBizLocation(), formData.getTimeZoneOffset());
				if (!recv.confirmPalletReceive(formData.getPalletUrn())) {
					formData
							.setLastError("Unable to confirm receive operation with backend!. Report to administrator");
					target = SHOW_PALLET_PAGE;
				} else {
					// receive confirmed
					log.finest("Receiving confirmed with backend for barcode ="
							+ formData.getPalletTag());
					// update received quantity in the session variable.
					OrderInfo previousOrder = (OrderInfo) request.getSession()
							.getAttribute(PREVIOUS_ORDER);

					if (previousOrder != null) {
						previousOrder.getReceivedPalletEPCs().add(
								formData.getPalletUrn());
					} else {
						log
								.severe("Previous Order information not found in session for tag ="
										+ formData.getPalletTag());
					}

				}
			} else {
				log
						.severe("No station ID supplied. Using default value HandHeld1");
				recv = new Receiving("HandHeld1", formData.getBizLocation(), formData.getTimeZoneOffset());
			}
		} catch (UseCaseException e) {
			e.printStackTrace();
		}
		log.exiting(getClass().getName(), "processConfirm", formData);
		return target;

	}

	protected String processPalletTagWithDummyData(FormData formData,
			ActionErrors errors) {
		String target = null;
		log.entering(getClass().getName(), "processPalletTag", formData); //$NON-NLS-1$

		/*
		 * this is too real for right now, but it works String barcode =
		 * formData.getSerialNumber(); encodingTypes type =
		 * AbstractBarCodeParser.getEncodingType(barcode);
		 * 
		 * if (type.equals(encodingTypes.SSCC)) { try { SSCC18BarCodeParser
		 * parser = new SSCC18BarCodeParser(barcode);
		 * 
		 * log.finest("Raw barcode = " + barcode); log.finest("companyCode = " +
		 * parser.getCompanyPrefix()); log.finest("serial number = " +
		 * parser.getSerialNumber()); log.finest("checkdigit = " +
		 * parser.getCheckDigit()); log.finest("EPC URN = " + parser.getEPC());
		 * 
		 * formData.setPalletUrn(parser.getEPC()); } catch (Exception e) {
		 * target=REDISPLAY; formData.setLastError("invalid pallet barcode"); } }
		 * else { target=REDISPLAY; formData.setLastError("invalid pallet
		 * barcode"); }
		 */
		// these are the same in testing but in production if the user scans a
		// pallet from a new order
		// then we have to put up a warning and either discard the new pallet
		// tag or continue processing
		// pallets under that order...
		String orderNum = "10000-004-009";
		formData.setOrderNumber(orderNum);
		formData.setLastOrderNum(orderNum);

		final int j = 0x0001;
		int cases = 0;
		int units = 0;

		String[] ndc = { "00000-000-00", "11111-111-11", "22222-222-22",
				"33333-333-33" };
		String[] cas = { "1", "3", "9", "12" };
		String[] unt = { "48", "150", "9", "144" };
		String[] des = { "Drug A 300mg 10 pills", "Drug B 300mg 20 pills",
				"Drug C 300mg 30 pills", "Drug E 300mg 40 pills" };

		String arr = "<p><table class=\"scanData\">"
				+ "<tr><th width=70%>NDC</th>" + "<th width=15%>Cases</th>"
				+ "<th width=15%>Units</th></tr>"
				+ "<tr><th colspan=3>Item Description</th></tr>";

		for (int i = 0; i < ndc.length; i++) {
			int k = i & j;
			if (k == 0) {
				arr = arr + "<tr class=\"even\"><td>";
			} else {
				arr = arr + "<tr><td>";
			}
			arr = arr + ndc[i] + "</td><td>" + cas[i] + "</td><td>" + unt[i]
					+ "</td></tr>";
			if (k == 0) {
				arr = arr + "<tr class=\"even\">";
			} else {
				arr = arr + "<tr>";
			}
			arr = arr + "<td colspan=3>" + des[i] + "</td></tr>";
			cases += Integer.parseInt(cas[i]);
			units += Integer.parseInt(unt[i]);
		}
		arr = arr + "</td></tr></table>";
		formData.setDisplayData(arr);

		formData.setCaseCount(cases);
		formData.setUnitCount(units);
		formData.setPalletNum(9);
		formData.setPalletCnt(12);

		log.exiting(getClass().getName(),
				"processPalletTag", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	protected ActionForward execute(ActionMapping mapping, FormData formData,
			HttpServletRequest request, HttpServletResponse response,
			ActionErrors errors) throws Exception {

		log.entering(getClass().getName(), "execute", formData); //$NON-NLS-1$

		ActionForward forward = new ActionForward(); // return value
		String target = null;
		String screen = formData.getScreen();
		String command = formData.getCommand();

		log.finest("Command =" + command);

		// Strip barcode of prefix
		formData.setSerialNumber(BarcodeUtils.stripBarcodePrefix(formData
				.getSerialNumber()));

		if (RoleValidation.validate(request, RoleDefs.RECEIVER) == false) {
			target = "GotoMainMenu";

		} else if (command.equals("GotPallet")) { //$NON-NLS-1$
			target = processPalletTag(formData, errors, request);
			PharmaUnitInfo unitInfo = new PharmaUnitInfo("urn:epc:id:1234");
			request.getSession().setAttribute("unit", unitInfo);
			if (target == null) {
				target = SHOW_PALLET_PAGE;
			} else if (target.equals(REDISPLAY)) {
				target = GET_PALLET_PAGE;
			}
		} else if (command.equals("Confirm")) { //$NON-NLS-1$
			if (screen.equalsIgnoreCase("ReceivingOrderWarning")) {
				target = processOrderWarningConfirm(formData, errors, request);
			} else {
				target = processConfirm(formData, errors, request);
			}

			if (target == null)
				target = GET_PALLET_PAGE;

		} else if (command.equals("Cancel")) { //$NON-NLS-1$
			target = GET_PALLET_PAGE;

		} else if (command.equals("MainMenu")) { //$NON-NLS-1$
			target = MAIN_MENU;
		}

		if (target != null) {
			forward = mapping.findForward(target);
			request.setAttribute(Constants.FORM_DATA_ATTR, formData);
		} else {
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.UNKNOWN_COMMAND_4, request.getRequestURI(),
					screen, command, this.getClass().getName()));
		}

		log.exiting(getClass().getName(),
				"execute", "target=" + target + "; forward=" + forward); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

		// Finish with
		return (forward);
	}

	/**
	 * Build the display string to render the barcode product information.
	 * 
	 * @param map
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String buildDisplayData(PharmaUnitInfo tagInfo,
			FormData formData, ActionErrors errors) {

		log.entering(getClass().getName(), "buildDisplayData", new Object[] {
				tagInfo, formData, errors });

		String target = null;

		if (tagInfo != null) {

			String display = "<p><table class=\"scanData\">"
					+ "<tr><th width=70%>NDC</th>" + "</tr>"
					+ "<tr><th colspan=3>Item Description</th></tr>";
			display = display + "<tr class=\"even\"><td>";

			display = display + tagInfo.getNDC() + "</td></tr>";

			display = display + "<td colspan=3>" + tagInfo.getProductName()
					+ "</td></tr>";

			display = display + "</td></tr></table>";
			formData.setDisplayData(display);
		} else {
			// nothing to display
			log.severe("Input parameter PharmaUnitInfo is null!");
			formData
					.setLastError("Unable to display tag information due to error. Please refer to logs");
			formData.setDisplayData("");
		}

		log.exiting(getClass().getName(), "buildDisplayData", target);
		return target;
	}

}
