package com.ibm.trackntrace.actions;

import java.util.ArrayList;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.ibm.trackntrace.forms.FormData;
import com.ibm.trackntrace.usecases.Shipping;
import com.ibm.trackntrace.usecases.UseCaseException;
import com.ibm.trackntrace.usecases.data.CaseInfo;
import com.ibm.trackntrace.usecases.data.PalletInfo;
import com.ibm.trackntrace.usecases.data.PharmaUnitInfo;
import com.ibm.trackntrace.utils.BarcodeUtils;
import com.ibm.trackntrace.utils.Constants;
import com.ibm.trackntrace.utils.RoleDefs;
import com.ibm.trackntrace.utils.RoleValidation;




public class ShippingAction extends BXActionBase {

	private static final String MAIN_MENU = "GotoMainMenu";

	private static final String GET_PALLET_PAGE = "ShippingScanPallet";

	private static final String GET_ORDER_PAGE = "ShippingGetOrder";

	private static final String PALLET_TAG_DETAILS_MAP = "ShippingTagDetailsMap";

	static {
		log = Logger.getLogger(Constants.LOG_SHIPPING);
	}

	protected String processPalletTagWithDummyData(FormData formData,
			ActionErrors errors) {
		String target = null;
		log.entering(getClass().getName(), "processPalletTag", formData); //$NON-NLS-1$

		String serial = formData.getSerialNumber();
		if (serial != null) {
			String tags[] = formData.getTagList();
			if (tags == null) {
				tags = new String[1];
				tags[0] = serial;
			} else {
				if (arrayDoesNotContain(tags, serial)) {
					String newList[] = new String[tags.length + 1];
					System.arraycopy(tags, 0, newList, 0, tags.length);
					newList[newList.length - 1] = serial;
					tags = newList;
					log.log(Level.INFO, "Tag added to list: " + serial
							+ " len now: " + newList.length);
				} else {
					log.log(Level.INFO, "Tag already in list: " + serial);
				}
			}
			formData.setTagList(tags);
			formData.setPalletTag(serial);
			formData.setNumTagsScanned(tags.length);
			formData.setSerialNumber("");
		}

		buildDisplayData(formData, errors);

		log.exiting(getClass().getName(),
				"processPalletTag", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	protected String processPalletTag(Map<String, PalletInfo>tagDetailsMap, FormData formData, ActionErrors errors,
			HttpServletRequest request) {
		String target = null;
		log.entering(getClass().getName(), "processPalletTag", formData); //$NON-NLS-1$

		// get current barcode scan
		String thisTag = formData.getSerialNumber();
		String thisUrn = null;

		if (thisTag != null) {
			// parse barcode to EPC URN
			thisUrn = BarcodeUtils.convertBarcodeToEPCpureID(thisTag, formData);
		} else {
			log.warning("Passed in barcode is null!");
			formData.setLastError("Null Barcode scanned in");
			target = REDISPLAY;
		}

		// get the counts and transfer order number from the backend

		try {
			String userAgentID = formData.getStation();
			Shipping shipping = null;
			PalletInfo palletInfo = null;

			if (userAgentID != null) {
				shipping = new Shipping(userAgentID, formData.getBizLocation(), 
						formData.getTimeZoneOffset());
			} else {
				log
						.severe("No station ID supplied. Using default value HandHeld1");
				formData.setLastError("No station set!");
				return target;
			}

			if (thisUrn != null){
				
				palletInfo = shipping.getPalletDetails(thisUrn); // order information 
			}	
				
			else {
				log.severe("URN is null for pallet barcode " + thisTag);
				formData.setLastError("Null URN for scanned barcode");
			}

			if (palletInfo != null) {
				
				//add this pallet to the session map
				
				tagDetailsMap.put(thisTag, palletInfo);

				// set pallet and count information in the form for display
				formData.setPalletTag(thisTag);
				formData.setPalletUrn(thisUrn);
				formData.setCaseCount(palletInfo.getCasesPerPallet());
				formData.setUnitCount(palletInfo.getUnitsPerPallet());
				formData.setNumTagsScanned(tagDetailsMap.size());

				// if we have cases and units display their information on the
				// GUI.
				if (palletInfo.getCasesPerPallet() >= 1
						&& palletInfo.getUnitsPerPallet() >= 1) {
					PharmaUnitInfo unit = palletInfo.getCases().get(0)
							.getUnits().get(0);

					if (unit != null) {
						this.buildDisplayData(tagDetailsMap, formData, errors);
					} else {
						formData
								.setLastError("Unable to display pallet information. Please report to administrator");
						log.severe("Unit information is null for pallet "
								+ formData.getPalletTag());
					}
				}
			}

		} catch (UseCaseException e) {
			e.printStackTrace();
			formData.setLastError("Backend error occured. Please report to Administrator");
		}

		log
				.exiting(
						getClass().getName(),
						"processPalletTag", "target=" + target + " formData =" + formData.toString()); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	protected String processConfirm(Map<String, PalletInfo>tagDetailsMap, FormData formData, ActionErrors errors) {
		
		String target = GET_ORDER_PAGE;
		log.entering(getClass().getName(), "processConfirm", formData); //$NON-NLS-1$

		if( tagDetailsMap != null){
			
			ArrayList<String> palletUrns = new ArrayList<String>();
			Object[] pallets = tagDetailsMap.values().toArray();
			
			for( int i=0; i< pallets.length; ++i){
				PalletInfo pallet = (PalletInfo) pallets[i];
				palletUrns.add(pallet.getPalletURN());
			}
			
			if( palletUrns.size() >= 1){
				try{
					Shipping shipping = new Shipping(formData.getStation(), formData.getBizLocation(), 
							formData.getTimeZoneOffset());
					
					
					if (shipping.confirmShipping(palletUrns, formData.getOrderNumber(), formData.getDestinationId())){
						target = GET_ORDER_PAGE;
					}
					else{
						formData.setLastError("Unable to confirm shipping with backend.");
						log.severe("Unable to confirm shipping with backend for order " + formData.getOrderNumber() + 
								" to destination = " + formData.getDestinationId() );
					}
					
				}catch( UseCaseException e){
					e.printStackTrace();
					formData.setLastError("Unable to confirm shipping with backend.");

					target = GET_PALLET_PAGE;
				}finally{
					formData.setSerialNumber("");
					formData.setNumTagsScanned(0);
					formData.setPalletTag("");
					tagDetailsMap.clear();
				}
			}
		}

		log.exiting(getClass().getName(), "processConfirm", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

	protected boolean processOrder(FormData formData, ActionErrors errors) {

		log.entering(getClass().getName(), "processOrder", formData); //$NON-NLS-1$

		// validate Order Number
		// No validation required at this point, as the order number is
		// maintained by the
		// client not RFIDIC.

		log.exiting(getClass().getName(), "processOrder"); //$NON-NLS-1$ //$NON-NLS-2$
		return true;
	}

	protected boolean processDestination(FormData formData, ActionErrors errors) {

		boolean result = false;
		log.entering(getClass().getName(), "processDestination", formData); //$NON-NLS-1$

		// validate destinationId entered

		try {
			Shipping shipping = new Shipping(formData.getStation(), formData.getBizLocation(), 
			formData.getTimeZoneOffset());

			result = shipping.validateDestination(formData.getDestinationId());

		} catch (UseCaseException e) {
			e.printStackTrace();
			formData
					.setLastError("Unable to validate destionation Id with backend!");
			result = false;
		}

		log.exiting(getClass().getName(),
				"processDestination", "result=" + result); //$NON-NLS-1$ //$NON-NLS-2$
		return result;
	}

	protected String processOrderInfo(FormData formData, ActionErrors errors) {
		String target = null;
		log.entering(getClass().getName(), "processOrderInfo", formData); //$NON-NLS-1$

		
		//strip any prefixes in case order number and destination were scanned in.
		formData.setOrderNumber(BarcodeUtils.stripBarcodePrefix(formData
				.getOrderNumber()));
		
		formData.setDestinationId(BarcodeUtils.stripBarcodePrefix(formData
				.getDestinationId()));
		String order = formData.getOrderNumber();
		String dest = formData.getDestinationId();

		if (order == null || order == "") {
			formData.setLastError("Order number is required.");
			target = "MISSING_ORDER_NUMBER";
		} else {
			processOrder(formData, errors);
		}
		if (target == null) {
			if (dest == null || dest == "") {
				formData.setLastError("Destination is required.");
				target = "MISSING_DESTINATION";
			} else {
				if( !processDestination(formData, errors)){
					formData.setLastError("Destination not found in backend");
					target = REDISPLAY;
				}
			}
		}

		log.exiting(getClass().getName(),
				"processOrderInfo", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}


	protected ActionForward execute(ActionMapping mapping, FormData formData,
			HttpServletRequest request, HttpServletResponse response,
			ActionErrors errors) throws Exception {
		ActionForward forward = new ActionForward(); // return value
		String target = null;
		String screen = formData.getScreen();
		String command = formData.getCommand();

		log.entering(getClass().getName(), "execute", formData); //$NON-NLS-1$

		// Strip barcode of prefix
		formData.setSerialNumber(BarcodeUtils.stripBarcodePrefix(formData
				.getSerialNumber()));

		// get the barcode details from the session
		Map<String, PalletInfo> tagDetailsMap = createPalletTagDetailsMapInSession(
				request, PALLET_TAG_DETAILS_MAP);

		log.finest("Command = " + command + " Screen =" + screen);

		if (RoleValidation.validate(request, RoleDefs.SHIPPER) == false) {
			target = "GotoMainMenu";

		} else if (command.equals("GotOrder")) { //$NON-NLS-1$
			target = processOrderInfo(formData, errors);
			if (target == null) {
				target = GET_PALLET_PAGE;
			} else if (target.equals(REDISPLAY)) {
				target = GET_ORDER_PAGE;
			}

		} else if (command.equals("GotPallet")) { //$NON-NLS-1$
			target = processPalletTag(tagDetailsMap, formData, errors, request);
			if (target == null) {
				target = GET_PALLET_PAGE;
			} else if (target.equals(REDISPLAY)) {
				target = GET_PALLET_PAGE;
			}

		} else if (command.equals("Confirm")) {
			target = processConfirm(tagDetailsMap,formData, errors);
			if (target == null) {
				target = GET_PALLET_PAGE;
			}
			

		} else if (command.equals("Undo")) { //$NON-NLS-1$
			target = processUndo(tagDetailsMap, formData, errors);
			
			if( target == null){
				target = GET_PALLET_PAGE;
			}

		} else if (command.equals("Cancel")) { //$NON-NLS-1$
			if( tagDetailsMap != null){
				tagDetailsMap.clear();
			}
			target = GET_ORDER_PAGE;

		} else if (command.equals("MainMenu")) { //$NON-NLS-1$
			target = MAIN_MENU;
		}

		if (target != null) {
			forward = mapping.findForward(target);
			request.setAttribute(Constants.FORM_DATA_ATTR, formData);
		} else {
			errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionError(
					Constants.UNKNOWN_COMMAND_4, request.getRequestURI(),
					screen, command, this.getClass().getName()));
		}

		log.exiting(getClass().getName(),
				"execute", "target=" + target + "; forward=" + forward); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

		// Finish with
		return (forward);
	}

	/**
	 * Build the display string to render the barcode product information.
	 * 
	 * @param map
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String buildDisplayData(Map<String, PalletInfo> map,
			FormData formData, ActionErrors errors) {

		log.entering(getClass().getName(), "buildDisplayData", new Object[] {
				map, formData, errors });

		String target = null;

		ArrayList<String> ndc = new ArrayList<String>();
		ArrayList<String> des = new ArrayList<String>();
		ArrayList<Integer> caseCount = new ArrayList<Integer>();
		ArrayList<Integer> unitCount = new ArrayList<Integer>();

		Object[] palletInfoArray = (Object[]) map.values().toArray();

		if (palletInfoArray.length != 0) {

			for (int i = 0; i < palletInfoArray.length; ++i) {
				PalletInfo pallet = (PalletInfo) palletInfoArray[i];
				if (pallet != null) {
					// get first case
					ArrayList<CaseInfo> cases = pallet.getCases();
					if (cases != null && cases.size() > 0) {
						CaseInfo aCase = cases.get(0);
						ArrayList<PharmaUnitInfo> units = aCase.getUnits();
						if (units != null && units.size() > 0) {
							PharmaUnitInfo unit = units.get(0);
							if (unit != null) {
								ndc.add(unit.getNDC());
								des.add(unit.getProductName());
							}
						}
					}
					caseCount.add(pallet.getCasesPerPallet());
					unitCount.add(pallet.getUnitsPerPallet());
				}
			}

			String display = "<p><table class=\"scanData\">"
					+ "<tr><th width=70%>NDC</th>" + "</tr>"
					+ "<tr><th width=70%>Cases</th>" + "</tr>"
					+ "<tr><th width=70%>Units</th>" + "</tr>"
					+ "<tr><th colspan=3>Item Description</th></tr>";
			final int j = 0x0001;

			for (int i = 0; i < palletInfoArray.length; i++) {
				int k = i & j;
				if (k == 0) {
					display = display + "<tr class=\"even\"><td>";
				} else {
					display = display + "<tr><td>";
				}
				display = display + ndc.get(i) + "</td></tr>";
				
				if (k == 0) {
					display = display + "<tr class=\"even\"><td>";
				} else {
					display = display + "<tr><td>";
				}
				display = display + caseCount.get(i) + "</td></tr>";
				
				if (k == 0) {
					display = display + "<tr class=\"even\"><td>";
				} else {
					display = display + "<tr><td>";
				}
				display = display + unitCount.get(i) + "</td></tr>";

				if (k == 0) {
					display = display + "<tr class=\"even\">";
				} else {
					display = display + "<tr>";
				}
				display = display + "<td colspan=3>" + des.get(i)
						+ "</td></tr>";
			}
			display = display + "</td></tr></table>";
			formData.setDisplayData(display);
		} else {
			// nothing to display
			log
					.warning("TagDetailsMap is empty! Setting form unit information display data to null");
			formData.setDisplayData("");
		}

		log.exiting(getClass().getName(), "buildDisplayData", target);
		return target;
	}
	
	/**
	 * Handles the undo button press on the GUI by removing the last item scanned.
	 * @param tagDetailsMap
	 * @param formData
	 * @param errors
	 * @return
	 */
	protected String processUndo(Map<String, PalletInfo> tagDetailsMap,
			FormData formData, ActionErrors errors) {
		
		String target = GET_PALLET_PAGE;
		String lastBarcode = null;
		log.entering(getClass().getName(), "processUndo", formData); //$NON-NLS-1$
		
		
		if( tagDetailsMap == null || formData == null){
			log.severe("Input arguments are not as expected. Null argument(s) passed in");
			return target;
		}
		//get the last barcode from the session map
		if(tagDetailsMap.size() != 0){
			//get the last entry in the map, this is a LinkedHashMap, so the keys are ordered by insertion
			Object[] barcodes = tagDetailsMap.keySet().toArray();
			if( barcodes.length > 0){
				lastBarcode = (String) barcodes[barcodes.length -1];
				if( lastBarcode != null){
					//remove the details from the session
					if (!this.removePalletBarCodeDetailsFromSession(tagDetailsMap, formData, lastBarcode, PALLET_TAG_DETAILS_MAP)) {
						target = REDISPLAY;
						return target;
					}	
					else{
						//get new barcode list
						Object[] newBarcodeList = tagDetailsMap.keySet().toArray();
						formData.setNumTagsScanned(tagDetailsMap.size());
						if( newBarcodeList.length > 0){
							formData.setPalletTag((String) newBarcodeList[newBarcodeList.length - 1]);
							//	change display to reflect the undo.
							buildDisplayData(tagDetailsMap, formData, errors);
						}
						else{
							//no more barcode left
							formData.setSerialNumber("");
							formData.setPalletTag("");
						}
					}
				}
			}
		}
		else{
			log.warning("Nothing to undo!");
			formData.setLastError("Nothing to undo!");
		}
	

		log.exiting(getClass().getName(), "processUndo", "target=" + target); //$NON-NLS-1$ //$NON-NLS-2$
		return target;
	}

}
