package com.ibm.trackntrace.usecases;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import com.ibm.premises.reusable.constants.Constants;
import com.ibm.trackntrace.utils.Constants.BusinessSteps;
import com.ibm.trackntrace.utils.Constants.Disposition;

/**
 * @author Sudhir Pucha
 * This class implements the Split pallet use case.
 *
 */
public class SplitPallet extends AbstractUseCase {

	/**
	 * Logger for this class. Log level can be adjusted from the WAS admin
	 * console
	 */
	private static Logger theLogger = Logger.getLogger(SplitPallet.class
			.getName());

	/**
	 *  class name
	 */
	private String clsName = SplitPallet.class.getName();

	/**
	 * @param locationID
	 */
	public SplitPallet(String locationID, String bizLoc, String tzOffset) {
		this.location = locationID;
		this.businessLocation= bizLoc;
		this.timeZoneOffset = tzOffset;
	}

	/**
	 * @param caseList
	 * @param newPalletUrn
	 * @return
	 * @throws UseCaseException
	 */
	public boolean confirmSplitPallet(ArrayList<String> caseList,
			String newPalletUrn) throws UseCaseException {

		methodName = "confirmSplitPallet";
		boolean result = false;
		

		theLogger.entering(clsName, methodName, new Object[] {
				caseList.toArray(), newPalletUrn });
		
		// get the old pallettag and disaggregate from it
		// aggreate to the new pallet

		if (caseList != null && newPalletUrn != null) {
			for (int i = 0; i < caseList.size(); ++i) {

				String urn = (String) caseList.get(i);

				String oldPalletUrn = getParent(urn, true);

				if (oldPalletUrn != null) {
					ArrayList<String> disaggrTagList = new ArrayList<String>();
					
					Map<String, String> metaData = new HashMap<String, String>();
					metaData.put(Constants.ObjectEventConstants.BIZSTEP_KEY_OUT,
							BusinessSteps.INSPECTING);
					metaData.put(Constants.ObjectEventConstants.DISPOSITION_KEY_OUT,
							Disposition.INACTIVE);
					metaData.put(Constants.ObjectEventConstants.READPOINT_KEY_OUT,
							this.location);
					
					if( this.businessLocation != null ){
						metaData.put(Constants.ObjectEventConstants.BIZLOCATION_KEY_OUT,
							this.businessLocation);
					}
					else{
						theLogger.warning("Business location information missing from WAS resource environment");
					}
					metaData.put(
							Constants.ObjectEventConstants.EVENT_TIME_ZONE_OFFSET_KEY_OUT,
							this.timeZoneOffset);
					metaData.put("PARENTTAG", oldPalletUrn);
					// disaggregate from the old pallet
					disaggrTagList.add(oldPalletUrn);
					disaggrTagList.add(urn);
					this.disaggregate(disaggrTagList, metaData);

					// aggregate to the new pallet
					disaggrTagList.remove(oldPalletUrn);
					disaggrTagList.add(newPalletUrn);
					
					metaData.clear();
					metaData.put(Constants.ObjectEventConstants.BIZSTEP_KEY_OUT,
							BusinessSteps.INSPECTING);
					metaData.put(Constants.ObjectEventConstants.DISPOSITION_KEY_OUT,
							Disposition.IN_PROGRESS);
					metaData.put(Constants.ObjectEventConstants.READPOINT_KEY_OUT,
							this.location);
					metaData.put(
							Constants.ObjectEventConstants.EVENT_TIME_ZONE_OFFSET_KEY_OUT,
							this.timeZoneOffset);
					if( this.businessLocation != null ){
						metaData.put(Constants.ObjectEventConstants.BIZLOCATION_KEY_OUT,
							this.businessLocation);
					}
					
					metaData.put("PARENTTAG", newPalletUrn);
					aggregate(disaggrTagList, metaData);
					
					result = true;
				} else {
					// unable to retrieve the parent urn
					theLogger.warning("The URN " + urn
							+ " returned a null parent!");
				}
			}
		}
		return result;
	}
}
