package com.ibm.trackntrace.usecases.util;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jms.Message;
import com.ibm.sensorevent.model.IBMSensorEvent;
import com.ibm.sensorevent.model.ISensorEvent;
import com.ibm.sensorevent.model.converter.XMLConverter;
import com.ibm.sensorevent.model.generic.SensorEventException;
import com.ibm.sensorevent.model.payload.EPCTag;
import com.ibm.sensorevent.model.payload.PassiveRFIDAggregatedTagReadPayload;
import com.ibm.sensorevent.model.payload.PassiveRFIDTagRead;
import com.ibm.trackntrace.usecases.AbstractUseCase;
import com.ibm.trackntrace.usecases.UseCaseException;

public class IBMSensorEventHelper {
	
	/**
	 * Logger for this class. Log level can be adjusted from the WAS admin
	 * console
	 */
	private static Logger theLogger = Logger.getLogger(AbstractUseCase.class
			.getName());

	/**
	 * variable that hold the class name of the object
	 */
	private static String clsName = AbstractUseCase.class.getName();

	/**
	 * variable to hold the method name
	 */
	protected static String methodName = null;
	
	/**
	 * @param message
	 * @return
	 */
	protected static ISensorEvent convertToSensorEvent(Message message){
		
		methodName = "publishToBus";
		theLogger.entering(clsName, methodName);
		// FIGURE THIS OUT...>>>>>>>>>>
		theLogger.exiting(clsName, methodName);
		return null;
	}
	


	/**
	 * @param readerID
	 * @param tagList
	 * @param eventType
	 * @return
	 * @throws UseCaseException
	 */
	public static ISensorEvent createIBMSensorEvent(String readerID,
			ArrayList<String> tagList, String eventType) throws UseCaseException {

		methodName = "createIBMSensorEvent";
		theLogger.entering(clsName, methodName, new Object[]{readerID, tagList, eventType});

		ISensorEvent ise = null;
		try {
			ise = IBMSensorEvent.getInstance();
			ise.getHeader().setSourceId(readerID);
			ise.getHeader().setEventType(eventType);

			PassiveRFIDAggregatedTagReadPayload payload = (PassiveRFIDAggregatedTagReadPayload) PassiveRFIDAggregatedTagReadPayload
					.getInstance(eventType);
			

			// Set this PassiveRFIDAggregatedTagReadPayload as the ISE payload
			ise.setPayload(payload);
		
			if( tagList != null) {
			// Add these tags to the payload
			addTagsToPayload(readerID, tagList, payload);
			}
			else{
				theLogger.warning("No TagList to add to payload");
			}

		} catch (SensorEventException e) {
			theLogger.severe("Caught SensorEventException Message = "
					+ e.getMessage());
			throw new UseCaseException(e.getMessage());
		}

		// Print the sensor event is log level is high enough
		if (Level.FINEST == theLogger.getLevel() && ise != null) {
			XMLConverter xconv = (XMLConverter) XMLConverter.getInstance();
			try {
				theLogger.finest("ise = \n" + xconv.toXMLString(ise));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		theLogger.exiting(clsName, methodName, ise);
		return ise;
	}
	/**
	 * @param urnTags
	 *            A List of Strings containing the tag values in EPC format
	 * @param payload
	 */
	protected static boolean addTagsToPayload(String readerID, ArrayList urnTags,
			PassiveRFIDAggregatedTagReadPayload payload) {

		methodName = "addTagsToPayload";
		theLogger.entering(clsName, methodName,new Object[]{readerID, urnTags,payload});

		PassiveRFIDTagRead tagread;
		EPCTag tag;
		String urnTag;

		for (int i = 0; i < urnTags.size(); i++) {
			try {
				tagread = (PassiveRFIDTagRead) PassiveRFIDTagRead.getInstance();
				tag = (EPCTag) EPCTag.getInstance();

				// setup tag
				urnTag = (String) urnTags.get(i);
				theLogger.fine("Adding tag '" + urnTag + "' to payload");

				tag.addStringAttribute(EPCTag.TAGURI, urnTag);

				// setup tag read
				tagread.setReaderID(readerID);
				tagread.setDiscovered(new Date().getTime());
				tagread.addTag(tag);

				// setup tag read payload
				payload.addTagRead(tagread);
			} catch (SensorEventException e) {
				// Exception adding this tag to the IBMSensorEvent payload
				e.printStackTrace();
				theLogger.exiting(clsName, methodName);
				return false;
			}
		}
		theLogger.exiting(clsName, methodName);
		return true;
	}
	
	@SuppressWarnings("unchecked")
	public static void addMetaDataToEvent(ISensorEvent sensorEvent, Map<String,String> metaData) throws UseCaseException {
		
		methodName = "addMetaDataToEvent";
		theLogger.entering(clsName, methodName, new Object[]{ sensorEvent, metaData });
		
		if( metaData != null  && sensorEvent != null){
			Iterator it = metaData.entrySet().iterator();
			while( it.hasNext()){
				Map.Entry<String, String> pairs = (Map.Entry<String, String>) it.next();
				theLogger.finest("MetaData Key = " + pairs.getKey() + " Value ="+ pairs.getValue());
				//add the key-val pairs to the sensor event data metadata section, these
				// pairs are used to constructed the EPCIS event.
				
				try {
					sensorEvent.getPayloadMetaData().addStringAttribute(pairs.getKey(), pairs.getValue());
				} catch (SensorEventException e){
					e.printStackTrace();
					theLogger.severe("Unable to Add Metadata to sensor event" + metaData);
					throw new UseCaseException(e.getMessage());
				}
			}
		}
		theLogger.exiting(clsName, methodName, sensorEvent);
		
		
	}
	
	@SuppressWarnings("unchecked")
	public static void addGroupDataToEvent(ISensorEvent sensorEvent, Map<String,String> metaData) throws UseCaseException {
		
		methodName = "addMetaDataToEvent";
		theLogger.entering(clsName, methodName, new Object[]{ sensorEvent, metaData });
		
		if( metaData != null  && sensorEvent != null){
			Iterator it = metaData.entrySet().iterator();
			while( it.hasNext()){
				Map.Entry<String, String> pairs = (Map.Entry<String, String>) it.next();
				theLogger.finest("MetaData Key = " + pairs.getKey() + " Value ="+ pairs.getValue());
				//add the key-val pairs to the sensor event data metadata section, these
				// pairs are used to constructed the EPCIS event.
				
				try {
					String groupName = sensorEvent.getHeader().getEventType();
					theLogger.finest("Group name = " + groupName);
					if( groupName != null)
						sensorEvent.getGroup(groupName).addStringAttribute(pairs.getKey(), pairs.getValue());

					else{
						theLogger.severe("Group name for sensorEvent is null!! " + sensorEvent);
					}
						
					
				} catch (SensorEventException e){
					e.printStackTrace();
					theLogger.severe("Unable to Add Metadata to sensor event" + metaData);
					throw new UseCaseException(e.getMessage());
				}
			}
		}
		theLogger.exiting(clsName, methodName, sensorEvent);
		
		
	}

}
