/*
 * IBM Confidential OCO Source Material
 *
 * Licensed Materials - Property of IBM
 * 5724-L17 WebSphere Premises Server
 * (C) Copyright IBM Corp. 2008  All rights reserved.
 *
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U. S. Copyright Office.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */

package com.ibm.trackntrace.usecases.util;

import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.ejb.EJBHome;
import javax.ejb.EJBLocalHome;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.sql.DataSource;
import javax.xml.rpc.Service;

/**
 * ResourceLocator
 */
public class ResourceLocator {

	@SuppressWarnings("unused")
	private static final String COPYRIGHT = "(C) COPYRIGHT International Business Machines Corp., 2008. "
		+ "All Rights Reserved * Licensed Materials - Property of IBM "
		+ "US Government Users Restricted Rights - Use, duplication or "
		+ "disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";
	private static ResourceLocator mySingleton;
	private Map<String, Object> cache;
	private InitialContext ic;
	
	protected ResourceLocator() throws Exception {
		this.cache = new HashMap<String, Object>();
		this.ic = new InitialContext();
	}
	
	public synchronized static ResourceLocator singleton() throws Exception {
		if (mySingleton == null) {
			mySingleton = new ResourceLocator();
		}
		
		return mySingleton;
	}
	
	public DataSource getDataSource(String name) throws Exception {
		DataSource ds = (DataSource) this.cache.get(name);
		if (ds == null) {
			ds = (DataSource) this.ic.lookup(name);
			this.cache.put(name, ds);
		}
		
		return ds;
	}
	
	public ResourceBundle getResourceBundle(String name) {
		ResourceBundle rb = (ResourceBundle) this.cache.get(name);
		if (rb == null) {
			rb = ResourceBundle.getBundle(name);
			this.cache.put(name, rb);
		}
		
		return rb;		
	}
	
	public EJBHome getEJBHome(String name, Class c) throws Exception {
		EJBHome home = (EJBHome) this.cache.get(name);
		if (home == null) {
			Object o = this.ic.lookup(name);
			home = (EJBHome) PortableRemoteObject.narrow(o, c);
			this.cache.put(name, home);
		}
		
		return home;
	}
	
	public EJBLocalHome getEJBLocalHome(String name) throws Exception {
		EJBLocalHome home = (EJBLocalHome) this.cache.get(name);
		if (home == null) {
			home = (EJBLocalHome) this.ic.lookup(name);
			this.cache.put(name, home);
		}
		
		return home;
	}
	public Service getService(String name) throws Exception {
		Service svc = (Service) this.cache.get(name);
		if (svc == null) {
			svc = (Service) this.ic.lookup(name);
			this.cache.put(name, svc);
		}

		return svc;
	}
}
