package com.ibm.trackntrace.utils;

import java.util.logging.Logger;


import com.ibm.annap.barcode.parser.AbstractBarCodeParser;
import com.ibm.annap.barcode.parser.BarCodeParserException;
import com.ibm.annap.barcode.parser.sgtin.PharmaSGTIN14BarCodeParser;
import com.ibm.annap.barcode.parser.sscc18.SSCC18BarCodeParser;
import com.ibm.trackntrace.forms.FormData;

public class BarcodeUtils {

	public static Logger log = Logger.getLogger(Constants.LOG_UTILS);

	public static String clsName = "BarcodeUtils";

	public static String stripBarcodePrefix(String barcode) {

		log.entering("clsName", "stripBarcodePrefix", barcode);

		String strippedBarcode = barcode;

		if (barcode != null
				&& barcode.contains(Constants.ONE_TWENTY_EIGHT_BARCODE_PREFIX)
				&& barcode.length() > Constants.ONE_TWENTY_EIGHT_BARCODE_PREFIX
						.length()) {
			strippedBarcode = barcode.substring(
					Constants.ONE_TWENTY_EIGHT_BARCODE_PREFIX.length(), barcode
							.length());
		} else if (barcode != null
				&& barcode.contains(Constants.DATAMATRIX_BARCODE_PREFIX)
				&& barcode.length() > Constants.DATAMATRIX_BARCODE_PREFIX
						.length()) {
			strippedBarcode = barcode.substring(
					Constants.DATAMATRIX_BARCODE_PREFIX.length(), barcode
							.length());
		} else if (barcode != null
				&& barcode.contains(Constants.ONE_TWENTY_EIGHT_BARCODE_PREFIX_VARIANT)
				&& barcode.length() > Constants.ONE_TWENTY_EIGHT_BARCODE_PREFIX_VARIANT
						.length()) {
			strippedBarcode = barcode.substring(
					Constants.ONE_TWENTY_EIGHT_BARCODE_PREFIX_VARIANT.length(), barcode
							.length());
		}

		log.exiting(clsName, "stripBarcodePrefix", strippedBarcode);
		return strippedBarcode;

	}

	public static String convertBarcodeToEPCpureID(String barcode,
			FormData formData) {

		log.entering(clsName, "convertBarcodeToEPCpureID", barcode);

		AbstractBarCodeParser.encodingTypes type = AbstractBarCodeParser
				.getEncodingType(barcode);
		String urn = null;
		if (type.equals(AbstractBarCodeParser.encodingTypes.SSCC)) {
			try {
				SSCC18BarCodeParser parser = new SSCC18BarCodeParser(barcode);

				log.finest("Raw barcode = " + barcode);
				log.finest("companyCode = " + parser.getCompanyPrefix());
				log.finest("serial number = " + parser.getSerialNumber());
				log.finest("checkdigit = " + parser.getCheckDigit());
				log.finest("EPC URN = " + parser.getEPC());

				urn = parser.getEPC();

			} catch (Exception e) {
				formData.setLastError("invalid pallet barcode");
				e.printStackTrace();
			}
		} else if (type.equals(AbstractBarCodeParser.encodingTypes.SGTIN)) {

			try {
				PharmaSGTIN14BarCodeParser parser = new PharmaSGTIN14BarCodeParser(
						barcode);
				log.finest("Raw barcode = " + barcode);
				log.finest("NDC = " + parser.getNDC());
				log.finest("serial number = " + parser.getSerialNumber());
				log.finest("pack type = " + parser.getPackagingTypeDigit());
				log.finest("EPC URN = " + parser.getEPC());
				urn = parser.getEPC();
			} catch (BarCodeParserException e) {
				formData.setLastError("invalid case or unit barcode");
				e.printStackTrace();
			}
		} else {
			formData.setLastError("unknown type of barcode " + barcode);
			log.severe("Unknown type of barcode, unable to parse barcode="
					+ barcode);
		}
		log.exiting(clsName, "convertBarcodeToEPCpureID", urn);
		return urn;
	}

}
