package com.ibm.trackntrace.utils;

import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import com.ibm.trackntrace.usecases.UseCaseException;


/**
 * @author Sudhir Pucha A SSCC URN is of the format
 *         urn:epc:id:sscc.companyprefix.serialnumber A SGTIN URN is of the
 *         format urn:epc:id:sgtin.companyprefix.itemref.serialnumber
 * 
 * An NDC is a 10 digit number that represent a drug. An NDC can be extracted
 * from an SGTIN tag.
 * 
 */
public class EPCPureIdURIDecoder {

	/**
	 * Logger for this class. Log level can be adjusted from the WAS admin
	 * console
	 */
	private static Logger theLogger = Logger
			.getLogger(EPCPureIdURIDecoder.class.getName());

	/**
	 * class name
	 */
	private String clsName = EPCPureIdURIDecoder.class.getName();

	private String companyPrefix = null;

	private String serialNumber = null;

	private String itemReference = null;

	private String ndc = null;

	private String urn = null;

	private String extensionDigit = null;

	private String packType = null;

	private String SSCC = "urn:epc:id:sscc";

	private String SGTIN = "urn:epc:id:sgtin";

	/**
	 * @param URN
	 * @throws UseCaseException
	 */
	public EPCPureIdURIDecoder(String URN) throws UseCaseException {

		theLogger.entering(clsName, "constructor", URN);

		this.urn = URN.toLowerCase();

		theLogger.finest("URN=" + this.urn);

		if (!this.urn.startsWith(SSCC) && !this.urn.startsWith(SGTIN)) {

			// unknown type of tag
			throw new UseCaseException("Unknown type of tag = " + URN);
		}

		parseEPC();
		System.out.println("Company Prefix=" + this.getCompanyPrefix()  +
				" Item Ref=" + this.getItemReference() +
				" Serial Number=" + this.getSerialNumber() +
				" NDC=" + this.getNdc());
		if(theLogger.getLevel() == java.util.logging.Level.FINEST){
			theLogger.info("Company Prefix=" + this.getCompanyPrefix()  +
							" Item Ref=" + this.getItemReference() +
							" Serial Number=" + this.getSerialNumber() +
							" NDC=" + this.getNdc());
		}

	}

	/**
	 * @return
	 */
	public String getCompanyPrefix() {
		return companyPrefix;
	}

	/**
	 * @param companyPrefix
	 */
	protected void setCompanyPrefix(String companyPrefix) {
		this.companyPrefix = companyPrefix;
	}

	/**
	 * @return
	 */
	public String getNdc() {
		return ndc;
	}

	/**
	 * @param ndc
	 */
	protected void setNdc(String ndc) {
		this.ndc = ndc;
	}

	/**
	 * @return
	 */
	public String getSerialNumber() {
		return serialNumber;
	}

	/**
	 * @param serialNumber
	 */
	protected void setSerialNumber(String serialNumber) {
		this.serialNumber = serialNumber;
	}

	/**
	 * @return
	 */
	/**
	 * @return
	 */
	public String getUrn() {
		return urn;
	}

	/**
	 * @param urn
	 */
	protected void setUrn(String urn) {
		this.urn = urn;
	}

	/**
	 * @return
	 */
	public String getItemReference() {
		return itemReference;
	}

	/**
	 * @param itemReference
	 */
	protected void setItemReference(String itemReference) {
		this.itemReference = itemReference;
	}

	/**
	 * @throws UseCaseException
	 */
	private void parseEPC() throws UseCaseException {

		StringTokenizer tokenzier = new StringTokenizer(this.getUrn(), ".");

		ArrayList<String> tokens = new ArrayList<String>();

		while (tokenzier.hasMoreTokens()) {
			String token = tokenzier.nextToken();
			theLogger.fine("token= " + token);
			tokens.add(token);
		}

		switch (tokens.size()) {

		case 2:
			// sscc - An SSCC tag has a company prefix and a serial number
			// portion
			String token1 = tokens.get(0);
			if (token1 != null && token1.length() > 1) {
				// now remove the urn:epc. .etc from this token to get the
				// companyPrefix
				int lastIndex = token1.lastIndexOf(":");
				this.companyPrefix = token1.substring(lastIndex+1, token1
						.length());
			}

			// now extract the serial number and extension digit

			String token2 = tokens.get(1);
			if (token2 != null && token2.length() > 1) {
				// the first digit of the serial number is the extension digit.
				this.extensionDigit = token2.substring(0, 1);
				this.serialNumber = token2.substring(1, token2.length());
			}
			break;

		case 3:
			// sgtin - An SGTIN tags has company prefix, item ref and serial
			// number portions
			token1 = tokens.get(0);

			if (token1 != null && token1.length() > 1) {
				// now remove the urn:epc. .etc from this token to get the
				// companyPrefix
				int lastIndex = token1.lastIndexOf(":");
				this.companyPrefix = token1.substring(lastIndex+1, token1
						.length());
			}

			// extract itemRef and packtype
			token2 = tokens.get(1);
			if (token2 != null && token2.length() > 1) {
				this.itemReference = token2;
				this.packType = token2.substring(0, 1);
			}

			// get serial number
			String token3 = tokens.get(2);
			if (token3 != null)
				this.serialNumber = token3;

			// construct ndc = (CompnayPrefix - first two digits) + ( ItemRef -
			// first digit )
			this.ndc = (this.companyPrefix.substring(2, this.companyPrefix
					.length()))
					+ (this.itemReference.substring(1, this.itemReference
							.length()));

			break;

		default:
			// unknown format.. must not be pure id form
			throw new UseCaseException(
					"Unknown format, expecting EPC Pure ID form. URN="
							+ this.urn);

		}
	}

	/**
	 * @return
	 */
	public String getExtensionDigit() {
		return extensionDigit;
	}

	/**
	 * @param extensionDigit
	 */
	protected void setExtensionDigit(String extensionDigit) {
		this.extensionDigit = extensionDigit;
	}

	/**
	 * @return
	 */
	public String getPackType() {
		return packType;
	}

	/**
	 * @param packType
	 */
	protected void setPackType(String packType) {
		this.packType = packType;
	}
	
	public static void main(String[] args) {
		
		try{
			String urn = null;
			urn = "urn:epc:id:sscc:0085412.1012345678";
			urn ="urn:epc:id:sgtin:038541.6210299.123456789";
			@SuppressWarnings("unused")
			EPCPureIdURIDecoder decoder = new EPCPureIdURIDecoder(urn);
		} catch( UseCaseException e){
			e.printStackTrace();
		}
		
	}

}
