var globals = {};
/**
 * Global variables related to LoadPageData.
 *
 * Many of these global variables are to avoid closure leakage for the onreadystatechange function.
 **/
globals.xmlhttp = null;         // Holder for the xmlhttp object.
globals.auto_refresh = false;   // Determines if a function should be called again after the page has been loaded.
globals.delay = 0;              // The amount of time to delay before calling the function when auto_refresh is set to true.
globals.function_name = null;   // The name of the function that will be invoked when auto_refresh is set to true.
globals.page_elem = null;       // The place to store the resulting data that was loaded.

/**
 * Found at
 * http://www.jibbering.com/faq/faq_notes/not_browser_detect.html
 **/
var getElementWithId;
// document.getElementById explodes the Intermec WM5 browser with an object exception, so, do this instead
if ((navigator.appVersion.lastIndexOf("IEMobile 6.8") != -1) || (document.getElementById)) {
    //alert("in navigator test");
    getElementWithId = function(id){
        var element = document.getElementById(id);
        if (element == null) {
            var elements = document.getElementsByName(id);
            if (elements != null) {
                element = elements[0];
            }
        }
        return element;
    }
    
} else if (document.all) {
    //alert("in document.all");
    /* Branch to use document.all on document.all only
       browsers. Requires that IDs are unique to the page
       and do not coincide with NAME attributes on other
       elements:-
    */
    getElementWithId = function(id){
        return document.all[id];
    }
    
} else if (document.layers) {
    //alert("in document.layers");
    /* Branch to use document.layers, but that will only work for CSS
       positioned elements. This function uses a recursive method
       to find positioned elements within positioned elements but most
       DOM nodes on document.layers browsers cannot be referenced at
       all. This function is expected to be called with only one
       argument, exactly like the over versions.
    */
    getElementWithId = function(id, baseLayers) {
       /* If the - baseLayers - parameter is not provided default
          its value to the document.layers collection of the main
          document:
       */
       baseLayers = baseLayers||document.layers;
       /* Assign the value of the property of the - baseLayers -
          object (possibly defaulted to the document.layers
          collection) with the property name corresponding to the
          - id - parameter to the local variable - obj:
       */
       var obj = baseLayers[id];
       /* If - obj - remains undefined (no element existed with the
          given - id -) try searching the indexed members of
          - baseLayers - to see if any of their layers collections
          contain the element with the corresponding - id:
       */
       if (!obj) { //Element not found
          /* Loop through the indexed members of - baseLayers: */
          for (var c = 0;c < baseLayers.length;c++) {
             if ((baseLayers[c])&&  //Object at index - c.
                 (baseLayers[c].document)&& //It has a - document.
                 /* And a layers collection on that document: */
                 (baseLayers[c].document.layers)) {
                  /* Recursively call this function passing the - id - as
                     the first parameter and the layers collection from
                     within the document found on the layer at index - c
                     - in - baseLayers - as the second parameter and
                     assign the result to the local variable - obj: 
                  */
                obj=getElementWithId(id,baseLayers[c].document.layers);
                /* If - obj - is now not null then we have found the
                   required element and can break out of the - for -
                   loop:
                */
                if (obj) {
                    break;
                }
             }
          }
       }
       /* If - obj - will type-convert to boolean true (it is not null
          or undefined) return it, else return null:
       */
       return obj||null;
    }
    
} else {
    //alert("in default");
    /* I've modified the default catch-all getElementWithId to
       use this mechanism to attempt to locate the element.
       Not sure how to test directly for this type of support.
    
       No appropriate element retrieval mechanism exists on
       this browser. So assign this function as a safe dummy.
       Values returned form calls to getElementWithId probably
       should be tested to ensure that they are non-null prior
       to use anyway so this branch always returns null:-
    */
    getElementWithId = function(id){
        var obj = null;
        try {
            obj = document[id];
            if (obj == null) {
                var i;
                for(i = 0; (i < document.forms.length) && (obj == null); ++i) {
                    var theForm = document.forms[i];
                    if (theForm.elements != null) {
                        obj = theForm.elements[id];
                    }
                }
            }
        } catch (e) {
            obj = null;
        }
        return obj;
    }
}

/*
Usage:-

var elRef = getElementWithId("ID_string");
if(elRef){
    ... //element was found
}else{
    ... //element could not be found.
}
*/

function findPageElement(id) {
    return getElementWithId(id);
}

function windowFocus() {
	if (navigator.appVersion.lastIndexOf("IEMobile 6.8") == -1) {
		window.focus();
	}
}

function focus(elementId)
{
    var element = getElementWithId(elementId);
    if (element != null) {
        element.focus();
    }
}

function getXmlHttpObject() {
    var xmlhttp = null;
    if (window.XMLHttpRequest) {
        xmlhttp = new XMLHttpRequest();
    } else if (window.ActiveXObject) {
        try {
            xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
        } catch (e) {
            try {
                xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
            } catch (e) {
                xmlhttp = null;
            }
        }
    } else if (window.createRequest) {
        try {
            xmlhttp = window.createRequest();
        } catch (e) {
            xmlhttp = false;
        }
    }

    return xmlhttp;
/*
    var xmlhttp=null;
    
    try {
        xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
    } catch (e) {
        try {
            xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
        } catch (E) {
            xmlhttp = false;
        }
    }


    if (!xmlhttp && typeof XMLHttpRequest !== 'undefined') {
        try {
            xmlhttp = new XMLHttpRequest();
        } catch (e) {
            xmlhttp = false;
        }
    }

    if (!xmlhttp && window.createRequest) {
        try {
            xmlhttp = window.createRequest();
        } catch (e) {
            xmlhttp = false;
        }
    }

    return xmlhttp;
    */
}

/*
    added by jjho

    http.open(mode,url,true); // Asynchronous
    or
    http.open(mode,url,false); // Synchronous
    
    http.open(mode,url,false);  // Synchronous request
    http.send(request);         // Locked until response returns
    alert(http.responseText);   // Then next line executes

    // GET mode, parameters in url
    webMethod("example.com/example.jsp?name1=value1", null, someMethod);

    // POST mode, parameters as request
    webMethod("example.com/example.jsp","name1=value1",someMethod);
*/
function webMethod(url, request, callback){
    var http = getXmlHttpObject();
    var mode = request?"POST":"GET";
    http.open(mode,url,true);
    if (mode === "POST" ) {
        http.setRequestHeader('Content-Type','application/x-www-form-urlencoded');
    }
    http.onreadystatechange = function() {
        if(http.readyState === 4) {
            callback(http.responseText);
        }
    };
    http.send(request);
}

function getDocumentBody(responseText) {
    return responseText;
}

function loadPageDataHelper() {
    if (globals.xmlhttp.readyState === 4) {
        globals.page_elem.innerHTML = globals.xmlhttp.responseText;
        globals.page_elem = null;
        if (globals.auto_refresh) {
            window.setTimeout(globals.function_name, globals.delay);
        }
    }
}
function loadPageDataPoller() {
    if (globals.xmlhttp.readyState === 4) {
        loadPageDataHelper();
    } else {
        window.setTimeout("loadPageDataPoller", 50);
    }
}
function loadPageData(url, element_name, auto_refresh, function_name, delay) {
    var timeout_id = undefined;
    var polling = false;

    if (!globals.xmllhttp) {
        globals.xmlhttp = getXmlHttpObject();
        if (!polling) {
            globals.xmlhttp.onreadystatechange = loadPageDataHelper;
        }
    }
    globals.auto_refresh = auto_refresh
    globals.function_name = function_name;
    globals.delay = delay;
    try{
        if (globals.xmlhttp) {
            var page_elem = findPageElement(element_name);
            if (page_elem !== null) {
                globals.page_elem = page_elem;
                globals.xmlhttp.open("GET", url, true);
                // Prevent IE from caching the page.
                globals.xmlhttp.setRequestHeader("If-Modified-Since", "Sat, 1 Jan 2000 00:00:00 GMT");
                globals.xmlhttp.send(null);
                if (polling) {
                    window.setTimeout("loadPageDataPoller", 50);
                }
            } else {
                alert("Cannot find a page element called '" + element_name + "'");
            }
        } else {
            alert("Cannot create an XMLHttpRequestObject.");
        }
    } catch (e) {
        alert("Caught exception '" + e + "' while trying to execute loadPageData(" + url + ", " + element_name + ", " + auto_refresh + ", " + function_name, + ", " + delay + ")");
    }
}
